#
# coding=utf-8
"""edist - Simple command line coordinate processing"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from email.utils import parseaddr

from upoints import (__version__, __author__)
from upoints.compat import mangle_repr_type


__doc__ += """.

edist operates on one, or more, locations specified in the following format
``[+-]DD.DD;[+-]DDD.DD``.  For example, a location string of ``52.015;-0.221``
would be interpreted as 52.015 degrees North by 0.221 degrees West.  Positive
values can be specified with a ``+`` prefix, but it isn't required.

For example::

    $ ./edist.py --sunrise --sunset --ascii '52.015;-0.221'
    $ ./edist.py --destination 20@45 -- '-52.015;0.221'

In the second example the locations are separated by ``--``, which stops
processing options and allows you to specify locations beginning with
a hyphen(such as anywhere in the Southern hemisphere).

.. note::
   In most shells the locations must be quoted because of the special nature of
   the semicolon.

.. currentmodule:: edist
.. moduleauthor:: `%s <mailto:%s>`__
""" % parseaddr(__author__)

# Pull the first paragraph from the docstring
USAGE = __doc__[:__doc__.find('\n\n', 100)].replace('``', "'").splitlines()[2:]
# Replace script name with optparse's substitution var, and rebuild string
USAGE = '\n'.join(USAGE).replace('edist', '%(prog)s')

import logging
import os
import sys

from operator import itemgetter

import aaargh

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser


from upoints import (point, utils)


# Pull the first paragraph from the docstring
USAGE = __doc__[:__doc__.find('\n\n', 100)].splitlines()[2:]
# Replace script name with optparse's substitution var, and rebuild string
USAGE = '\n'.join(USAGE).replace('edist', '%(prog)s')

EPILOG = 'Please report bugs at https://github.com/JNRowe/upoints/'

APP = aaargh.App(description=USAGE, epilog=EPILOG)


class LocationsError(ValueError):
    """Error object for data parsing error.

    .. versionadded:: 0.6.0

    .. attribute:: function

       Function where error is raised.

    .. attribute:: data

       Location number and data

    """
    def __init__(self, function=None, data=None):
        """Initialise a new ``LocationsError`` object.

        :param str function: Function where error is raised
        :param tuple data: Location number and data

        """
        super(LocationsError, self).__init__()
        self.function = function
        self.data = data

    def __str__(self):
        """Pretty printed error string.

        :rtype: ``str``
        :return: Human readable error string

        """
        if self.function:
            return 'More than one location is required for %s.' % self.function
        elif self.data:
            return 'Location parsing failure in location %i %r.' % self.data
        else:
            return 'Invalid location data.'


class NumberedPoint(point.Point):
    """Class for representing locations from command line.

    .. seealso::

       :class:`upoints.point.Point`.

    .. versionadded:: 0.6.0

    .. attribute:: name

       A name for location, or its position on the command line

    .. attribute:: units

       Unit type to be used for distances

    """

    __slots__ = ('name')

    def __init__(self, latitude, longitude, name, units='km'):
        """Initialise a new ``NumberedPoint`` object.

        :param float latitude: Location's latitude
        :param float longitude: Location's longitude
        :param str name: Location's name or command line position
        :param str units: Unit type to be used for distances

        """
        super(NumberedPoint, self).__init__(latitude, longitude, units)

        self.name = name

    def __format__(self, format_spec='dd'):
        """Extended pretty printing for location strings.

        :param str format_spec: Coordinate formatting system to use
        :rtype: ``str``
        :return: Human readable string representation of ``NumberedPoint``
            object
        :raise ValueError: Unknown value for ``format_spec``

        """
        return super(NumberedPoint, self).__format__('dm')


@mangle_repr_type
class NumberedPoints(point.Points):
    """Class for representing a group of :class:`NumberedPoint` objects.

    .. versionadded:: 0.6.0

    """

    def __init__(self, locations=None, format='dd', verbose=True,
                 config_locations=None, units='km'):
        """Initialise a new ``NumberedPoints`` object.

        :type locations: ``list`` of ``str`` objects
        :param locations: Location identifiers
        :param str format: Coordinate formatting system to use
        :param bool verbose: Whether to generate verbose output
        :param dict config_locations: Locations imported from user's config
            file
        :param str units: Unit type to be used for distances

        """
        super(NumberedPoints, self).__init__()
        self.format = format
        self.verbose = verbose
        self._config_locations = config_locations
        self.units = units
        if locations:
            self.import_locations(locations, config_locations)

    def __repr__(self):
        """Self-documenting string representation.

        :rtype: ``str``
        :return: String to recreate ``NumberedPoints`` object

        """
        return utils.repr_assist(self, {'locations': self[:]})

    def import_locations(self, locations, config_locations):
        """Import locations from arguments.

        :type locations: ``list`` of ``str``
        :param locations: Location identifiers
        :param dict config_locations: Locations imported from user's config
            file

        """
        for number, location in enumerate(locations):
            if config_locations and location in config_locations:
                latitude, longitude = config_locations[location]
                self.append(NumberedPoint(latitude, longitude, location,
                                          self.units))
            else:
                try:
                    data = utils.parse_location(location)
                    if data:
                        latitude, longitude = data
                    else:
                        latitude, longitude = utils.from_grid_locator(location)
                    self.append(NumberedPoint(latitude, longitude, number + 1,
                                              self.units))
                except ValueError:
                    raise LocationsError(data=(number, location))

    def display(self, locator):
        """Pretty print locations.

        :param str locator: Accuracy of Maidenhead locator output

        """
        for location in self:
            if self.format == 'locator':
                output = location.to_grid_locator(locator)
            else:
                output = format(location, self.format)
            if self.verbose:
                print('Location %s is %s' % (location.name, output))
            else:
                print(output)

    def distance(self):
        """Calculate distances between locations.

        """
        distances = list(super(NumberedPoints, self).distance())
        leg_msg = ['Location %s to %s is %i', ]
        total_msg = ['Total distance is %i', ]
        if self.units == 'sm':
            leg_msg.append('miles')
            total_msg.append('miles')
        elif self.units == 'nm':
            leg_msg.append('nautical miles')
            total_msg.append('nautical miles')
        else:
            leg_msg.append('kilometres')
            total_msg.append('kilometres')
        if self.verbose:
            for number, distance in enumerate(distances):
                print(' '.join(leg_msg) % (self[number].name,
                                           self[number + 1].name,
                                           distance))
            if len(distances) > 1:
                print(' '.join(total_msg) % sum(distances))
        else:
            print(sum(distances))

    def bearing(self, mode, string):
        """Calculate bearing/final bearing between locations.

        :param str mode: Type of bearing to calculate
        :param bool string: Use named directions

        """
        bearings = getattr(super(NumberedPoints, self), mode)()
        if string:
            bearings = map(utils.angle_to_name, bearings)
        else:
            bearings = ['%i°' % bearing for bearing in bearings]
        if mode == 'bearing':
            verbose_fmt = 'Location %s to %s is %s'
        else:
            verbose_fmt = 'Final bearing from location %s to %s is %s'
        for number, bearing in enumerate(bearings):
            if self.verbose:
                print(verbose_fmt % (self[number].name, self[number + 1].name,
                                     bearing))
            else:
                print(bearing)

    def range(self, distance):
        """Test whether locations are within a given range of the first.

        :param float distance: Distance to test location is within

        """
        test_location = self[0]
        for location in self[1:]:
            in_range = test_location.__eq__(location, distance)
            if self.verbose:
                text = ['Location %s is', ]
                if not in_range:
                    text.append('not')
                text.append('within %i')
                if self.units == 'sm':
                    text.append('miles')
                elif self.units == 'nm':
                    text.append('nautical miles')
                else:
                    text.append('kilometres')
                text.append('of location %s')
                print(' '.join(text) % (location.name, distance, self[0].name))
            else:
                print(in_range)

    def destination(self, distance, bearing, locator):
        """Calculate destination locations for given distance and bearings.

        :param float distance: Distance to travel
        :param float bearing: Direction of travel
        :param str locator: Accuracy of Maidenhead locator output

        """
        destinations = super(NumberedPoints, self).destination(bearing,
                                                               distance)
        for location, destination in zip(self, destinations):
            if self.format == 'locator':
                output = destination.to_grid_locator(locator)
            else:
                output = format(location, self.format)
            if self.verbose:
                print('Destination from location %s is %s' % (location.name,
                                                              output))
            else:
                print(output)

    def sun_events(self, mode):
        """Calculate sunrise/sunset times for locations.

        :param str mode: Sun event to display

        """
        mode_str = mode.capitalize()
        times = getattr(super(NumberedPoints, self), mode)()
        for location, time in zip(self, times):
            if self.verbose:
                if time:
                    print('%s at %s UTC in location %s' % (mode_str, time,
                                                           location.name))
                else:
                    print("The sun doesn't %s at location %s on this date"
                          % (mode_str[3:], location.name))
            else:
                print(time)

    def flight_plan(self, speed, time):
        """Output the flight plan corresponding to the given locations.

        .. todo:: Description

        :param float speed: Speed to use for elapsed time calculation
        :param str time: Time unit to use for output

        """
        if len(self) == 1:
            raise LocationsError('flight_plan')
        if self.verbose:
            print('WAYPOINT,BEARING[°],DISTANCE[%s],ELAPSED_TIME[%s],'
                  'LATITUDE[d.dd],LONGITUDE[d.dd]' % (self.units, time))
        legs = [(0, 0), ] + list(self.inverse())
        for leg, loc in zip(legs, self):
            if leg == (0, 0):
                print('%s,,,,%f,%f' % (loc.name, loc.latitude, loc.longitude))
            else:
                leg_speed = '%.1f' % (leg[1] / speed) if speed != 0 else ''
                print('%s,%i,%.1f,%s,%f,%f'
                      % (loc.name, leg[0], leg[1], leg_speed, loc.latitude,
                         loc.longitude))
        if self.verbose:
            overall_distance = sum(map(itemgetter(1), legs))
            direct_distance = self[0].distance(self[-1])
            if speed == 0:
                speed_marker = '#'
                overall_speed = ''
                direct_speed = ''
            else:
                speed_marker = ''
                overall_speed = '%.1f' % (overall_distance / speed)
                direct_speed = '%.1f' % (direct_distance / speed)
            print('-- OVERALL --%s,,%.1f,%s,,'
                  % (speed_marker, overall_distance, overall_speed))
            print('-- DIRECT --%s,%i,%.1f,%s,,'
                  % (speed_marker, self[0].bearing(self[-1]), direct_distance,
                     direct_speed))


@APP.cmd(help='pretty print the location(s)')
@APP.cmd_arg('-l', '--locator', choices=('square', 'subsquare', 'extsquare'),
             default='subsquare',
             help='accuracy of Maidenhead locator output')
@APP.cmd_arg('location', nargs='+', help='Locations to operate on')
def display(args):
    args.locations.display(args.locator)


@APP.cmd(help='calculate the distance between locations')
@APP.cmd_arg('location', nargs='+', help='Locations to operate on')
def distance(args):
    args.locations.distance()


@APP.cmd(help='calculate the initial bearing between locations')
@APP.cmd_arg('-g', '--string', action='store_true',
             help='display named bearings')
@APP.cmd_arg('location', nargs='+', help='Locations to operate on')
def bearing(args):
    args.locations.bearing('bearing', args.string)


@APP.cmd(name='final-bearing',
         help='calculate the final bearing between locations')
@APP.cmd_arg('-g', '--string', action='store_true',
             help='display named bearings')
@APP.cmd_arg('location', nargs='+', help='Locations to operate on')
def final_bearing(args):
    args.locations.bearing('final_bearing', args.string)


@APP.cmd(help='calculate whether locations are within a given range')
@APP.cmd_arg('-d', '--distance', type=float, help='range radius')
@APP.cmd_arg('location', nargs='+', help='Locations to operate on')
def range(args):
    args.locations.range(args.distance)


@APP.cmd(help='calculate the destination for a given distance and bearing')
@APP.cmd_arg('-l', '--locator', choices=('square', 'subsquare', 'extsquare'),
             default='subsquare',
             help='accuracy of Maidenhead locator output')
@APP.cmd_arg('-d', '--distance', required=True, type=float,
             help='distance from start point')
@APP.cmd_arg('-b', '--bearing', required=True, type=float,
             help='bearing from start point')
@APP.cmd_arg('location', nargs='+', help='Locations to operate on')
def destination(args):
    args.locations.destination(args.distance, args.bearing, args.locator)


@APP.cmd(help='calculate the sunrise time for a given location')
@APP.cmd_arg('location', nargs='+', help='Locations to operate on')
def sunrise(args):
    args.locations.sun_events('sunrise')


@APP.cmd(help='calculate the sunset time for a given location')
@APP.cmd_arg('location', nargs='+', help='Locations to operate on')
def sunset(args):
    args.locations.sun_events('sunset')


@APP.cmd(name='flight-plan',
         help='calculate the flight plan corresponding to locations (route)')
@APP.cmd_arg('-s', '--speed', default=0, type=float,
             help='speed to calculate elapsed time')
@APP.cmd_arg('-t', '--time', choices=('h', 'm', 's'),
             help='display time in hours, minutes or seconds')
@APP.cmd_arg('location', nargs='+', help='Locations to operate on')
def flight_plan(args):
    args.locations.flight_plan(args.speed, args.time)


def read_locations(filename):
    """Pull locations from a user's config file.

    :param str filename: Config file to parse
    :rtype: ``dict``
    :return: List of locations from config file

    """
    data = ConfigParser()
    data.read(filename)
    if not data.sections():
        logging.debug('Config file %r is empty' % filename)
        return {}

    locations = {}
    for name in data.sections():
        if data.has_option(name, 'locator'):
            latitude, longitude = utils.from_grid_locator(data.get(name,
                                                                   'locator'))
        else:
            latitude = data.getfloat(name, 'latitude')
            longitude = data.getfloat(name, 'longitude')
        locations[name] = (latitude, longitude)
    return locations


def read_csv(filename):
    """Pull locations from a user's CSV file.

    Read gpsbabel_'s CSV output format

    .. _gpsbabel: http://www.gpsbabel.org/

    :param str filename: CSV file to parse (STDIN if '-')
    :rtype: ``tuple`` of ``dict`` and ``list``
    :return: List of locations as ``str`` objects

    """
    if filename == '-':
        filename = sys.stdin
    field_names = ('latitude', 'longitude', 'name')
    data = utils.prepare_csv_read(filename, field_names, skipinitialspace=True)
    index = 0
    locations = {}
    args = []
    for row in data:
        index += 1
        name = '%02i:%s' % (index, row['name'])
        locations[name] = (row['latitude'], row['longitude'])
        args.append(name)
    return locations, args


def main():
    """Main script handler.

    :rtype: ``int``
    :return: 0 for success, >1 error code

    """
    logging.basicConfig(format='%(asctime)s %(levelname)s:%(message)s')

    APP.arg('--version', action='version',
            version='%%(prog)s v%s' % __version__)

    APP.arg('-v', '--verbose', action='store_true', dest='verbose',
            default=True,
            help='produce verbose output')
    APP.arg('-q', '--quiet', action='store_false', dest='verbose',
            help='output only results and errors')

    APP.arg('--config-file', metavar='~/.edist.conf',
            default=os.path.expanduser('~/.edist.conf'),
            help='config file to read custom locations from')
    APP.arg('--csv-file',
            help='CSV file (gpsbabel format) to read route/locations from '
                 "('-' for STDIN)")

    APP.arg('-o', '--format', choices=('dms', 'dm', 'dd', 'locator'),
            default='dms',
            help='produce output in dms, dm, d format or Maidenhead locator')
    APP.arg('-g', '--string', action='store_true',
            help='display named bearings')
    APP.arg('-u', '--units', choices=('km', 'sm', 'nm'), metavar='km',
            default='km',
            help='display distances in kilometres(default), statute miles or '
                 'nautical miles')
    APP.arg('-t', '--time', choices=('h', 'm', 's'), metavar='h', default='h',
            help='display time in hours(default), minutes or seconds')

    args = APP._parser.parse_args()
    func = args._func

    if args.csv_file:
        config_locations, args.location = read_csv(args.csv_file)
    else:
        config_locations = read_locations(args.config_file)

    try:
        args.locations = NumberedPoints(args.location, args.format,
                                        args.verbose, config_locations,
                                        args.units)
    except LocationsError as error:
        APP._parser.error(error)

    try:
        return func(args)
    except RuntimeError as error:
        APP._parser.error(error)



#
# coding=utf-8
"""test_trigpoints - Test trigpoints support"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from unittest import TestCase

from expecter import expect

from upoints.trigpoints import (Trigpoint, Trigpoints)


class TestTrigpoint(TestCase):
    def test___repr__(self):
        expect(repr(Trigpoint(52.010585, -0.173443, 97.0, 'Bygrave'))) == \
            "Trigpoint(52.010585, -0.173443, 97.0, 'Bygrave', None)"

    def test___str__(self):
        expect(str(Trigpoint(52.010585, -0.173443, 97.0))) == \
            """52°00'38"N, 000°10'24"W alt 97m"""
        expect(str(Trigpoint(52.010585, -0.173443, 97.0, 'Bygrave'))) == \
            """Bygrave (52°00'38"N, 000°10'24"W alt 97m)"""

    def test___format__(self):
        expect(format(Trigpoint(52.010585, -0.173443, 97.0), 'dd')) == \
            """N52.011°; W000.173° alt 97m"""
        expect(format(Trigpoint(52.010585, -0.173443, 97.0), 'dm')) ==\
            """52°00.64'N, 000°10.41'W alt 97m"""


class TestTrigpoints(TestCase):
    def test_import_locations(self):
        markers = Trigpoints(open('tests/data/trigpoints'))
        data = ['%s - %s' % (k, v) for k, v in sorted(markers.items())]
        expect(data[0]) == \
            """500936 - Broom Farm (52°03'57"N, 000°16'53"W alt 37m)"""
        expect(data[1]) == \
            """501097 - Bygrave (52°00'38"N, 000°10'24"W alt 97m)"""
        expect(data[2]) == \
            """505392 - Sish Lane (51°54'39"N, 000°11'11"W alt 136m)"""
        markers = Trigpoints(open('tests/data/southern_trigpoints'))
        expect(str(markers[1])) == \
            """FakeLand (48°07'23"S, 000°07'23"W alt 12m)"""
        markers = Trigpoints(open('tests/data/broken_trigpoints'))
        data = ['%s - %s' % (k, v) for k, v in sorted(markers.items())]
        expect(data[0]) == \
            """500968 - Brown Hill Nm  See The Heights (53°38'23"N, 001°39'34"W)"""
        expect(data[1]) == \
            """501414 - Cheriton Hill Nm  See Paddlesworth (51°06'03"N, 001°08'33"E)"""



#
# coding=utf-8
"""test_utils - Test utility functions"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime

from unittest import TestCase

from expecter import expect

from upoints.point import Point
from upoints.trigpoints import Trigpoint
from upoints.utils import (FileFormatError, Timestamp, TzOffset,
                           angle_to_distance, angle_to_name, calc_radius,
                           distance_to_angle, dump_xearth_markers,
                           from_grid_locator, from_iso6709, parse_location,
                           prepare_csv_read, prepare_read, prepare_xml_read,
                           sun_rise_set, to_dd, to_dms, to_grid_locator,
                           to_iso6709, value_or_empty)


class TestFileFormatError(TestCase):
    with expect.raises(FileFormatError, 'Unsupported data format.'):
        raise FileFormatError
    with expect.raises(
        FileFormatError,
        ("Incorrect data format, if you're using a file downloaded "
         'from test site please report this to James Rowe '
         '<jnrowe@gmail.com>')):
        raise FileFormatError('test site')


def test_value_or_empty():
    expect(value_or_empty(None)) == ''
    expect(value_or_empty('test')) == 'test'


def test_prepare_read():
    expect(prepare_read(open('tests/data/real_file'))) == \
        ['This is a test file-type object\n']
    test_list = ['This is a test list-type object', 'with two elements']
    expect(prepare_read(test_list)) == \
        ['This is a test list-type object', 'with two elements']
    expect(prepare_read(open('tests/data/real_file'), 'read')) == \
        'This is a test file-type object\n'


def test_prepare_csv_read():
    expect(list(prepare_csv_read(open('tests/data/real_file.csv'),
                                 ('type', 'bool', 'string')))) == \
        [{'bool': 'true', 'type': 'file', 'string': 'test'}]
    test_list = ['James,Rowe', 'ell,caro']
    expect(list(prepare_csv_read(test_list, ('first', 'last')))) == \
        [{'last': 'Rowe', 'first': 'James'}, {'last': 'caro', 'first': 'ell'}]


def test_prepare_xml_read():
    data = prepare_xml_read(open('tests/data/real_file.xml'))
    expect(data.find('tag').text) == 'This is a test file-type object'
    test_list = ['<xml>', '<tag>This is a test list</tag>', '</xml>']
    expect(prepare_xml_read(test_list).find('tag').text) == \
        'This is a test list'


def test_to_dms():
    expect(to_dms(52.015)) == (52, 0, 54.0)
    expect(to_dms(-0.221)) == (0, -13, -15.600000000000023)
    expect(to_dms(-0.221, style='dm')) == (0, -13.26)
    with expect.raises(ValueError, 'Unknown style type None'):
        to_dms(-0.221, style=None)


def test_to_dd():
    expect('%.3f' % to_dd(52, 0, 54)) == '52.015'
    expect('%.3f' % to_dd(0, -13, -15)) == '-0.221'
    expect('%.3f' % to_dd(0, -13.25)) == '-0.221'


def test_angle_to_name():
    expect(angle_to_name(0)) == 'North'
    expect(angle_to_name(360)) == 'North'
    expect(angle_to_name(45)) == 'North-east'
    expect(angle_to_name(292)) == 'West'
    expect(angle_to_name(293)) == 'North-west'
    expect(angle_to_name(0, 4)) == 'North'
    expect(angle_to_name(360, 16)) == 'North'
    expect(angle_to_name(45, 4, True)) == 'NE'
    expect(angle_to_name(292, 16, True)) == 'WNW'


class TestTzOffset(TestCase):
    def test__offset(self):
        expect(TzOffset('+00:00').utcoffset()) == datetime.timedelta(0)
        expect(TzOffset('-00:00').utcoffset()) == datetime.timedelta(0)
        expect(TzOffset('+05:30').utcoffset()) == datetime.timedelta(0, 19800)
        expect(TzOffset('-08:00').utcoffset()) == datetime.timedelta(-1, 57600)

    def test___repr__(self):
        expect(repr(TzOffset('+00:00'))) == "TzOffset('+00:00')"
        expect(repr(TzOffset('-00:00'))) == "TzOffset('+00:00')"
        expect(repr(TzOffset('+05:30'))) == "TzOffset('+05:30')"
        expect(repr(TzOffset('-08:00'))) == "TzOffset('-08:00')"


class TestTimestamp(TestCase):
    def parse_isoformat(self):
        expect(Timestamp.parse_isoformat('2008-02-06T13:33:26+0000')) == \
            Timestamp(2008, 2, 6, 13, 33, 26, tzinfo=TzOffset('+00:00'))
        expect(Timestamp.parse_isoformat('2008-02-06T13:33:26+00:00')) == \
            Timestamp(2008, 2, 6, 13, 33, 26, tzinfo=TzOffset('+00:00'))
        expect(Timestamp.parse_isoformat('2008-02-06T13:33:26+05:30')) == \
            Timestamp(2008, 2, 6, 13, 33, 26, tzinfo=TzOffset('+05:30'))
        expect(Timestamp.parse_isoformat('2008-02-06T13:33:26-08:00')) == \
            Timestamp(2008, 2, 6, 13, 33, 26, tzinfo=TzOffset('-08:00'))
        expect(Timestamp.parse_isoformat('2008-02-06T13:33:26z')) == \
            Timestamp(2008, 2, 6, 13, 33, 26, tzinfo=TzOffset('+00:00'))


def test_from_iso6709_wiki_page():
    # The following tests are from the examples contained in the wikipedia
    # ISO 6709 page(http://en.wikipedia.org/wiki/ISO_6709)

    #  Atlantic Ocean
    expect(from_iso6709('+00-025/')) == (0.0, -25.0, None)
    #  France
    expect(from_iso6709('+46+002/')) == (46.0, 2.0, None)
    #  Paris
    expect(from_iso6709('+4852+00220/')) == \
        (48.86666666666667, 2.3333333333333335, None)
    #  Eiffel Tower
    expect(from_iso6709('+48.8577+002.295/')) == (48.8577, 2.295, None)
    #  Mount Everest
    expect(from_iso6709('+27.5916+086.5640+8850/')) == \
        (27.5916, 86.564, 8850.0)
    #  North Pole
    expect(from_iso6709('+90+000/')) == (90.0, 0.0, None)
    #  Pacific Ocean
    expect(from_iso6709('+00-160/')) == (0.0, -160.0, None)
    #  South Pole
    expect(from_iso6709('-90+000+2800/')) == (-90.0, 0.0, 2800.0)
    #  United States
    expect(from_iso6709('+38-097/')) == (38.0, -97.0, None)
    #  New York City
    expect(from_iso6709('+40.75-074.00/')) == (40.75, -74.0, None)
    #  Statue of Liberty
    expect(from_iso6709('+40.6894-074.0447/')) == (40.6894, -74.0447, None)


def test_from_iso6709_location_page():
    # The following tests are from the Latitude, Longitude and Altitude format
    # for geospatial information page
    # (http://www.w3.org/2005/Incubator/geo/Wiki/LatitudeLongitudeAltitude)

    #  Mount Everest
    expect(from_iso6709('+27.5916+086.5640+8850/')) == \
        (27.5916, 86.564, 8850.0)
    #  South Pole
    expect(from_iso6709('-90+000+2800/')) == (-90.0, 0.0, 2800.0)
    #  New York City
    expect(from_iso6709('+40.75-074.00/')) == (40.75, -74.0, None)
    #  Mount Fuji
    expect(from_iso6709('+352139+1384339+3776/')) == \
        (35.36083333333333, 138.7275, 3776.0)
    #  Tokyo Tower
    expect(from_iso6709('+35.658632+139.745411/')) == \
        (35.658632, 139.745411, None)

    with expect.raises(ValueError, "Incorrect format for longitude '+1'"):
        from_iso6709('+35.658632+1/')


def test_to_iso6709_wiki_page():
    # The following tests are from the examples contained in the wikipedia
    # ISO 6709 page(http://en.wikipedia.org/wiki/ISO_6709)

    #  Atlantic Ocean
    expect(to_iso6709(0.0, -25.0, None, 'd')) == '+00-025/'
    #  France
    expect(to_iso6709(46.0, 2.0, None, 'd')) == '+46+002/'
    #  Paris
    expect(to_iso6709(48.866666666666667, 2.3333333333333335, None, 'dm')) == \
        '+4852+00220/'

    #FIXME
    #  The following test is skipped, because the example from wikipedia
    #  uses differing precision widths for latitude and longitude. Also,
    #  that degree of formatting flexibility is not seen anywhere else and
    #  adds very little.
    #  Eiffel Tower
    #expect(to_iso6709(48.857700000000001, 2.2949999999999999, None)) == \
    #    '+48.8577+002.295/'

    #  Mount Everest
    expect(to_iso6709(27.5916, 86.563999999999993, 8850.0)) == \
        '+27.5916+086.5640+8850/'
    #  North Pole
    expect(to_iso6709(90.0, 0.0, None, 'd')) == '+90+000/'
    #  Pacific Ocean
    expect(to_iso6709(0.0, -160.0, None, 'd')) == '+00-160/'
    #  South Pole
    expect(to_iso6709(-90.0, 0.0, 2800.0, 'd')) == '-90+000+2800/'
    #  United States
    expect(to_iso6709(38.0, -97.0, None, 'd')) == '+38-097/'
    #  New York City
    expect(to_iso6709(40.75, -74.0, None, precision=2)) == '+40.75-074.00/'
    #  Statue of Liberty
    expect(to_iso6709(40.689399999999999, -74.044700000000006, None)) == \
        '+40.6894-074.0447/'


def test_to_iso6709_location_page():
    # The following tests are from the Latitude, Longitude and Altitude format
    # for geospatial information page
    # (http://www.w3.org/2005/Incubator/geo/Wiki/LatitudeLongitudeAltitude)

    #  Mount Everest
    expect(to_iso6709(27.5916, 86.563999999999993, 8850.0)) == \
        '+27.5916+086.5640+8850/'
    #  South Pole
    expect(to_iso6709(-90.0, 0.0, 2800.0, 'd')) == '-90+000+2800/'
    #  New York City
    expect(to_iso6709(40.75, -74.0, None, precision=2)) == '+40.75-074.00/'
    #  Mount Fuji
    expect(to_iso6709(35.360833333333332, 138.72749999999999, 3776.0,
                      'dms')) == '+352139+1384339+3776/'
    #  Tokyo Tower
    expect(to_iso6709(35.658631999999997, 139.74541099999999, None,
                      precision=6)) == '+35.658632+139.745411/'


def test_angle_to_distance():
    expect('%.3f' % angle_to_distance(1)) == '111.125'
    expect('%i' % angle_to_distance(360, 'imperial')) == '24863'
    expect('%i' % angle_to_distance(1.0 / 60, 'nautical')) == '1'

    with expect.raises(ValueError, "Unknown units type 'baseless'"):
        '%i' % angle_to_distance(10, 'baseless')


def test_distance_to_angle():
    expect('%.3f' % round(distance_to_angle(111.212))) == '1.000'
    expect('%i' % round(distance_to_angle(24882, 'imperial'))) == '360'
    expect('%i' % round(distance_to_angle(60, 'nautical'))) == '1'


def test_from_grid_locator():
    expect('%.3f, %.3f' % from_grid_locator('BL11bh16')) == '21.319, -157.904'
    expect('%.3f, %.3f' % from_grid_locator('IO92va')) == '52.021, -0.208'
    expect('%.3f, %.3f' % from_grid_locator('IO92')) == '52.021, -1.958'


def test_to_grid_locator():
    expect(to_grid_locator(21.319, -157.904, 'extsquare')) == 'BL11bh16'
    expect(to_grid_locator(52.021, -0.208, 'subsquare')) == 'IO92va'
    expect(to_grid_locator(52.021, -1.958)) == 'IO92'


def test_parse_location():
    expect('%.3f;%.3f' % parse_location('52.015;-0.221')) == '52.015;-0.221'
    expect('%.3f;%.3f' % parse_location('52.015,-0.221')) == '52.015;-0.221'
    expect('%.3f;%.3f' % parse_location('52.015 -0.221')) == '52.015;-0.221'
    expect('%.3f;%.3f' % parse_location('52.015N 0.221W')) == '52.015;-0.221'
    expect('%.3f;%.3f' % parse_location('52.015 N 0.221 W')) == '52.015;-0.221'
    expect('%.3f;%.3f' % parse_location('52d00m54s N 0d13m15s W')) == \
        '52.015;-0.221'
    expect('%.3f;%.3f' % parse_location('52d0m54s N 000d13m15s W')) == \
        '52.015;-0.221'
    expect('%.3f;%.3f' % parse_location('''52d0'54" N 000d13'15" W''')) == \
        '52.015;-0.221'


def test_sun_rise_set():
    expect(sun_rise_set(52.015, -0.221, datetime.date(2007, 6, 15))) == \
        datetime.time(3, 40)
    expect(sun_rise_set(52.015, -0.221, datetime.date(2007, 6, 15),
                        'set')) == \
        datetime.time(20, 22)
    expect(sun_rise_set(52.015, -0.221, datetime.date(2007, 6, 15),
                        timezone=60)) == datetime.time(4, 40)
    expect(sun_rise_set(52.015, -0.221, datetime.date(2007, 6, 15), 'set',
                        60)) == datetime.time(21, 22)
    expect(sun_rise_set(52.015, -0.221, datetime.date(1993, 12, 11))) == \
        datetime.time(7, 58)
    expect(sun_rise_set(52.015, -0.221, datetime.date(1993, 12, 11),
                        'set')) == datetime.time(15, 49)
    expect(sun_rise_set(89, 0, datetime.date(2007, 12, 21))) == None
    expect(sun_rise_set(52.015, -0.221, datetime.date(2007, 2, 21))) == \
        datetime.time(7, 4)
    expect(sun_rise_set(52.015, -0.221, datetime.date(2007, 1, 21))) == \
        datetime.time(7, 56)


def sun_events():
    expect(sun_events(52.015, -0.221, datetime.date(2007, 6, 15))) == \
        (datetime.time(3, 40), datetime.time(20, 22))
    expect(sun_events(52.015, -0.221, datetime.date(2007, 6, 15), 60)) == \
        (datetime.time(4, 40), datetime.time(21, 22))
    expect(sun_events(52.015, -0.221, datetime.date(1993, 12, 11))) == \
        (datetime.time(7, 58), datetime.time(15, 49))
    expect(sun_events(52.015, -0.221, datetime.date(2007, 6, 15))) == \
        (datetime.time(3, 40), datetime.time(20, 22))
    expect(sun_events(40.638611, -73.762222, datetime.date(2007, 6, 15))) == \
        (datetime.time(9, 23), datetime.time(0, 27))
    expect(sun_events(49.016666, -2.5333333, datetime.date(2007, 6, 15))) == \
        (datetime.time(4, 5), datetime.time(20, 15))
    expect(sun_events(35.549999, 139.78333333,
                      datetime.date(2007, 6, 15))) == \
        (datetime.time(19, 24), datetime.time(9, 57))


def sun_events_civil():
    expect(sun_events(52.015, -0.221, datetime.date(2007, 6, 15),
                      zenith='civil')) == \
        (datetime.time(2, 51), datetime.time(21, 11))
    expect(sun_events(40.638611, -73.762222, datetime.date(2007, 6, 15),
                      zenith='civil')) == \
        (datetime.time(8, 50), datetime.time(1, 0))
    expect(sun_events(49.016666, -2.5333333, datetime.date(2007, 6, 15),
                      zenith='civil')) == \
        (datetime.time(3, 22), datetime.time(20, 58))
    expect(sun_events(35.549999, 139.78333333, datetime.date(2007, 6, 15),
                      zenith='civil')) == \
        (datetime.time(18, 54), datetime.time(10, 27))


def sun_events_nautical():
    expect(sun_events(52.015, -0.221, datetime.date(2007, 6, 15),
                      zenith='nautical')) == \
        (datetime.time(1, 32), datetime.time(22, 30))
    expect(sun_events(40.638611, -73.762222, datetime.date(2007, 6, 15),
                      zenith='nautical')) == \
        (datetime.time(8, 7), datetime.time(1, 44))
    expect(sun_events(49.016666, -2.5333333, datetime.date(2007, 6, 15),
                      zenith='nautical')) == \
        (datetime.time(2, 20), datetime.time(22, 0))
    expect(sun_events(35.549999, 139.78333333, datetime.date(2007, 6, 15),
                      zenith='nautical')) == \
        (datetime.time(18, 17), datetime.time(11, 5))


def sun_events_astronomical():
    expect(sun_events(52.015, -0.221, datetime.date(2007, 6, 15),
                      zenith='astronomical')) == (None, None)
    expect(sun_events(40.638611, -73.762222, datetime.date(2007, 6, 15),
                      zenith='astronomical')) == \
        (datetime.time(7, 14), datetime.time(2, 36))
    expect(sun_events(49.016666, -2.5333333, datetime.date(2007, 6, 15),
                      zenith='astronomical')) == (None, None)
    expect(sun_events(35.549999, 139.78333333, datetime.date(2007, 6, 15),
                      zenith='astronomical')) == \
        (datetime.time(17, 34), datetime.time(11, 48))


def test_dump_xearth_markers():
    markers = {
        500936: Trigpoint(52.066035, -0.281449, 37.000000, 'Broom Farm'),
        501097: Trigpoint(52.010585, -0.173443, 97.000000, 'Bygrave'),
        505392: Trigpoint(51.910886, -0.186462, 136.000000, 'Sish Lane')
    }
    data = dump_xearth_markers(markers)
    expect(data[0]) == '52.066035 -0.281449 "500936" # Broom Farm, alt 37m'
    expect(data[1]) == '52.010585 -0.173443 "501097" # Bygrave, alt 97m'
    expect(data[2]) == '51.910886 -0.186462 "505392" # Sish Lane, alt 136m'

    data = dump_xearth_markers(markers, 'name')
    expect(data[0]) == '52.066035 -0.281449 "Broom Farm" # 500936, alt 37m'
    expect(data[1]) == '52.010585 -0.173443 "Bygrave" # 501097, alt 97m'
    expect(data[2]) == '51.910886 -0.186462 "Sish Lane" # 505392, alt 136m'

    with expect.raises(ValueError, "Unknown name type 'falseKey'"):
        dump_xearth_markers(markers, 'falseKey')

    points = {
        'Broom Farm': Point(52.066035, -0.281449),
        'Bygrave': Point(52.010585, -0.173443),
        'Sish Lane': Point(51.910886, -0.186462)
    }
    data = dump_xearth_markers(points)
    expect(data[0]) == '52.066035 -0.281449 "Broom Farm"'
    expect(data[1]) == '52.010585 -0.173443 "Bygrave"'
    expect(data[2]) == '51.910886 -0.186462 "Sish Lane"'


def test_calc_radius():
    expect(calc_radius(52.015)) == 6375.166025311857
    expect(calc_radius(0)) == 6335.438700909687
    expect(calc_radius(90)) == 6399.593942121543
    expect(calc_radius(52.015, 'FAI sphere')) == 6371.0
    expect(calc_radius(0, 'Airy (1830)')) == 6335.022178542022
    expect(calc_radius(90, 'International')) == 6399.936553871439



#
# coding=utf-8
"""test_point - Test point support"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime
import math

from unittest import TestCase

from expecter import expect

from upoints import utils
from upoints.point import (KeyedPoints, Point, Points, TimedPoint, TimedPoints)


class TestPoint(TestCase):
    def test___init__(self):
        test = Point(math.pi / 4, math.pi / 2, angle='radians')
        expect(test.latitude) == 45
        expect(test.longitude) == 90

        test = Point((50, 20, 10), (-1, -3, -12))
        expect('%.3f' % test.latitude) == '50.336'
        expect('%.3f' % test.longitude) == '-1.053'

        with expect.raises(ValueError, 'Unknown angle type None'):
            Point(52.015, -0.221, angle=None)
        with expect.raises(ValueError, 'Invalid latitude value -92'):
            Point(-92, -0.221)
        with expect.raises(ValueError, 'Invalid longitude value 185'):
            Point(52.015, 185)
        with expect.raises(ValueError, 'Unknown units type None'):
            Point(52.015, -0.221, units=None)

    def test___dict__(self):
        home = Point(52.015, -0.221)
        expect(sorted(home.__dict__.items())) == \
            [('_angle', 'degrees'), ('_latitude', 52.015),
             ('_longitude', -0.221), ('_rad_latitude', 0.9078330104248505),
             ('_rad_longitude', -0.0038571776469074684), ('timezone', 0),
             ('units', 'metric')]

        class Test(Point):
            __slots__ = ('TEST', )

            def __init__(self, latitude, longitude):
                super(Test, self).__init__(latitude, longitude)
                self.TEST = 'tested'

        a = Test(52.015, -0.221)
        expect(sorted(a.__dict__.items())) == \
            [('TEST', 'tested'), ('_angle', 'degrees'),
             ('_latitude', 52.015), ('_longitude', -0.221),
             ('_rad_latitude', 0.9078330104248505),
             ('_rad_longitude', -0.0038571776469074684), ('timezone', 0),
             ('units', 'metric')]

    def test___repr__(self):
        expect(repr(Point(52.015, -0.221))) == \
            "Point(52.015, -0.221, 'metric', 'degrees', 0)"

    def test___str__(self):
        expect(str(Point(52.015, -0.221))) == 'N52.015°; W000.221°'

    def test___format__(self):
        expect(format(Point(52.015, -0.221), 'dm')) == \
            "52°00.90'N, 000°13.26'W"
        expect(format(Point(52.015, -0.221), 'dms')) == \
            """52°00'54"N, 000°13'15"W"""
        expect(format(Point(33.9400, -118.4000), 'dms')) == \
            """33°56'23"N, 118°24'00"W"""
        expect(format(Point(52.015, -0.221), 'locator')) == 'IO92'

    def test___unicode__(self):
        expect(str(Point(52.015, -0.221))) == 'N52.015°; W000.221°'

    def test___eq__(self):
        expect(Point(52.015, -0.221)) == Point(52.015, -0.221)

    def test___ne__(self):
        expect(Point(52.015, -0.221)) != Point(52.6333, -2.5)

    def test_to_grid_locator(self):
        home = Point(52.015, -0.221)
        expect(home.to_grid_locator('extsquare')) == 'IO92va33'
        expect(home.to_grid_locator('subsquare')) == 'IO92va'
        expect(home.to_grid_locator()) == 'IO92'

    def test_distance(self):
        home = Point(52.015, -0.221)
        dest = Point(52.6333, -2.5)
        expect('%i kM' % home.distance(dest)) == '169 kM'
        expect('%i kM' % home.distance(dest, method='sloc')) == '169 kM'

        with expect.raises(ValueError, "Unknown method type 'Invalid'"):
            home.distance(dest, method='Invalid')

        start = Point(36.1200, -86.6700)
        dest = Point(33.9400, -118.4000)
        expect('%i kM' % start.distance(dest)) == '2884 kM'
        start.units = 'imperial'
        expect('%i mi' % start.distance(dest)) == '1792 mi'
        start.units = 'nautical'
        expect('%i nmi' % start.distance(dest)) == '1557 nmi'
        start.units = 'metric'
        expect('%i kM' % start.distance(dest, method='sloc')) == '2884 kM'

    def test_bearing(self):
        expect(int(Point(52.015, -0.221).bearing(Point(52.6333, -2.5)))) == 294
        expect(int(Point(52.6333, -2.5).bearing(Point(52.015, -0.221)))) == 113
        expect(int(Point(36.1200, -86.6700).bearing(Point(33.9400, -118.4000)))) == \
            274
        expect(int(Point(33.9400, -118.4000).bearing(Point(36.1200, -86.6700)))) == \
            76
        expect(Point(52.015, -0.221).bearing(Point(52.6333, -2.5),
                                             format='string')) == 'North-west'

    def test_midpoint(self):
        expect(Point(52.015, -0.221).midpoint(Point(52.6333, -2.5))) == \
            Point(52.329631405407014, -1.3525368605590993, 'metric', 'degrees',
                  0)
        expect(Point(36.1200, -86.6700).midpoint(Point(33.9400, -118.4000))) == \
            Point(36.08239491900365, -102.75217370539663, 'metric', 'degrees',
                  0)

    def test_final_bearing(self):
        expect(int(Point(52.015, -0.221).final_bearing(Point(52.6333, -2.5)))) == 293
        expect(int(Point(52.6333, -2.5).final_bearing(Point(52.015, -0.221)))) == \
            114
        expect(int(Point(36.1200, -86.6700).final_bearing(Point(33.9400, -118.4000)))) == \
            256
        expect(int(Point(33.9400, -118.4000).final_bearing(Point(36.1200, -86.6700)))) == \
            94
        expect(Point(52.015, -0.221).bearing(Point(52.6333, -2.5),
                                             format='string')) == 'North-west'

    def test_destination(self):
        expect(Point(52.015, -0.221).destination(294, 169)) == \
            Point(52.611638750214745, -2.509374081952352, 'metric', 'degrees',
                  0)
        home = Point(52.015, -0.221, 'imperial')
        expect(home.destination(294, 169 / utils.STATUTE_MILE)) == \
            Point(52.611638750214745, -2.509374081952352, 'metric', 'degrees',
                  0)
        home = Point(52.015, -0.221, 'nautical')
        expect(home.destination(294, 169 / utils.NAUTICAL_MILE)) == \
            Point(52.611638750214745, -2.509374081952352, 'metric', 'degrees',
                  0)
        expect(Point(36.1200, -86.6700).destination(274, 2885)) == \
            Point(33.6872799137609, -118.32721842114393, 'metric', 'degrees',
                  0)

    def test_sunrise(self):
        date = datetime.date(2007, 6, 15)
        expect(Point(52.015, -0.221).sunrise(date)) == datetime.time(3, 40)
        expect(Point(52.6333, -2.5).sunrise(date)) == datetime.time(3, 45)
        expect(Point(36.1200, -86.6700).sunrise(date)) == datetime.time(10, 29)
        expect(Point(33.9400, -118.4000).sunrise(date)) == datetime.time(12, 41)

    def test_sunset(self):
        date = datetime.date(2007, 6, 15)
        expect(Point(52.015, -0.221).sunset(date)) == datetime.time(20, 22)
        expect(Point(52.6333, -2.5).sunset(date)) == datetime.time(20, 35)
        expect(Point(36.1200, -86.6700).sunset(date)) == datetime.time(1, 5)
        expect(Point(33.9400, -118.4000).sunset(date)) == datetime.time(3, 6)

    def test_sun_events(self):
        date = datetime.date(2007, 6, 15)
        expect(Point(52.015, -0.221).sun_events(date)) == \
            (datetime.time(3, 40), datetime.time(20, 22))
        expect(Point(52.6333, -2.5).sun_events(date)) == \
            (datetime.time(3, 45), datetime.time(20, 35))
        expect(Point(36.1200, -86.6700).sun_events(date)) == \
            (datetime.time(10, 29), datetime.time(1, 5))
        expect(Point(33.9400, -118.4000).sun_events(date)) == \
            (datetime.time(12, 41), datetime.time(3, 6))

    def test_inverse(self):
        bearing, dist = Point(52.015, -0.221).inverse(Point(52.6333, -2.5))
        expect(int(bearing)) == 294
        expect(int(dist)) == 169


class TestPoints(TestCase):
    def setUp(self):
        self.locs = Points(['52.015;-0.221', '52.168;0.040', '52.855;0.657'],
                           parse=True)

    def test___repr__(self):
        locations = [Point(0, 0)] * 4
        expect(repr(Points(locations))) == \
            ("Points([Point(0.0, 0.0, 'metric', 'degrees', 0), "
             "Point(0.0, 0.0, 'metric', 'degrees', 0), "
             "Point(0.0, 0.0, 'metric', 'degrees', 0), "
             "Point(0.0, 0.0, 'metric', 'degrees', 0)], "
             "False, 'metric')")

    def test_import_locations(self):
        locations = Points()
        locations.import_locations(['0;0', '52.015 -0.221'])
        expect(repr(locations)) == \
            ("Points([Point(0.0, 0.0, 'metric', 'degrees', 0), "
             "Point(52.015, -0.221, 'metric', 'degrees', 0)], "
             "False, 'metric')")

    def test_distance(self):
        expect('%.3f' % sum(self.locs.distance())) == '111.632'

    def test_bearing(self):
        expect(['%.3f' % x for x in self.locs.bearing()]) == \
            ['46.242', '28.416']

    def test_final_bearing(self):
        expect(['%.3f' % x for x in self.locs.final_bearing()]) == \
            ['46.448', '28.906']

    def test_inverse(self):
        expect(list(self.locs.inverse())) == \
            [(46.24239319802467, 24.629669163425465),
             (28.41617384845358, 87.00207583308533)]

    def test_midpoint(self):
        expect(list(self.locs.midpoint())) == \
            [Point(52.09157204324692, -0.09072375391429187, 'metric',
                   'degrees', 0),
             Point(52.51190105089283, 0.3460886030865466, 'metric',
                   'degrees', 0)]

    def test_range(self):
        expect(list(self.locs.range(Point(52.015, -0.221), 20))) == \
            [Point(52.015, -0.221, 'metric', 'degrees', 0)]

    def test_destination(self):
        expect(list(self.locs.destination(42, 240))) == \
            [Point(53.59560782169536, 2.2141813683976777, 'metric', 'degrees',
                   0),
             Point(53.74846914951471, 2.4840382137470614, 'metric', 'degrees',
                   0),
             Point(54.43483380445103, 3.1418347849815293, 'metric', 'degrees',
                   0)]

    def test_sunrise(self):
        expect(list(self.locs.sunrise(datetime.date(2008, 5, 2)))) == \
            [datetime.time(4, 28), datetime.time(4, 26), datetime.time(4, 21)]

    def test_sunset(self):
        expect(list(self.locs.sunset(datetime.date(2008, 5, 2)))) == \
            [datetime.time(19, 28), datetime.time(19, 27),
             datetime.time(19, 27)]

    def test_sun_events(self):
        expect(list(self.locs.sun_events(datetime.date(2008, 5, 2)))) == \
            [(datetime.time(4, 28), datetime.time(19, 28)),
             (datetime.time(4, 26), datetime.time(19, 27)),
             (datetime.time(4, 21), datetime.time(19, 27))]

    def test_to_grid_locator(self):
        expect(list(self.locs.to_grid_locator('extsquare'))) == \
            ['IO92va33', 'JO02ae40', 'JO02hu85']
        expect(list(self.locs.to_grid_locator('subsquare'))) == \
            ['IO92va', 'JO02ae', 'JO02hu']


class TestTimedPoints(TestCase):
    def speed(self):
        locations = TimedPoints()
        locations.extend([
            TimedPoint(52.015, -0.221,
                       time=datetime.datetime(2008, 7, 28, 16, 38)),
            TimedPoint(52.168, 0.040,
                       time=datetime.datetime(2008, 7, 28, 18, 38)),
            TimedPoint(52.855, 0.657,
                       time=datetime.datetime(2008, 7, 28, 19, 17)),
        ])
        expect(map(lambda s: '%.3f' % s, locations.speed())) == \
            ['12.315', '133.849']


class TestKeyedPoints(TestCase):
    def setUp(self):
        self.locs = KeyedPoints([('home', '52.015;-0.221'),
                                 ('Carol', '52.168;0.040'),
                                 ('Kenny', '52.855;0.657')],
                                parse=True)

    def test_import_locations(self):
        locations = KeyedPoints()
        locations.import_locations([('prime', '0;0'),
                                    ('home', '52.015 -0.221')])
        expect(locations) == \
            KeyedPoints({'prime': Point(0.0, 0.0, 'metric', 'degrees', 0),
                         'home': Point(52.015, -0.221, 'metric', 'degrees',
                                       0)},
                        False, 'metric')

    def test_distance(self):
        expect('%.3f' % sum(self.locs.distance(('home', 'Carol', 'Kenny')))) == '111.632'

    def test_bearing(self):
        expect(['%.3f' % x for x in self.locs.bearing(('home', 'Carol', 'Kenny'))]) == \
            ['46.242', '28.416']

    def test_final_bearing(self):
        expect(['%.3f' % x for x in self.locs.final_bearing(('home', 'Carol', 'Kenny'))]) == \
            ['46.448', '28.906']

    def test_inverse(self):
        expect(list(self.locs.inverse(('home', 'Carol', 'Kenny')))) == \
            [(46.24239319802467, 24.629669163425465),
             (28.41617384845358, 87.00207583308533)]

    def test_midpoint(self):
        expect(list(self.locs.midpoint(('home', 'Carol', 'Kenny')))) == \
            [Point(52.09157204324692, -0.09072375391429187, 'metric',
                   'degrees', 0),
             Point(52.51190105089283, 0.3460886030865466, 'metric', 'degrees',
                   0)]

    def test_range(self):
        expect(list(self.locs.range(Point(52.015, -0.221), 20))) == \
            [('home', Point(52.015, -0.221, 'metric', 'degrees', 0))]

    def test_destination(self):
        expect(sorted(self.locs.destination(42, 240))) == \
            [('Carol', Point(53.74846914951471, 2.4840382137470614, 'metric',
                             'degrees', 0)),
             ('Kenny', Point(54.43483380445103, 3.1418347849815293, 'metric',
                             'degrees', 0)),
             ('home', Point(53.59560782169536, 2.2141813683976777, 'metric',
                            'degrees', 0))]

    def test_sunrise(self):
        expect(sorted(self.locs.sunrise(datetime.date(2008, 5, 2)))) == \
            [('Carol', datetime.time(4, 26)),
             ('Kenny', datetime.time(4, 21)),
             ('home', datetime.time(4, 28))]

    def test_sunset(self):
        expect(sorted(self.locs.sunset(datetime.date(2008, 5, 2)))) == \
            [('Carol', datetime.time(19, 27)),
             ('Kenny', datetime.time(19, 27)),
             ('home', datetime.time(19, 28))]

    def test_sun_events(self):
        expect(sorted(self.locs.sun_events(datetime.date(2008, 5, 2)))) == \
            [('Carol', (datetime.time(4, 26), datetime.time(19, 27))),
             ('Kenny', (datetime.time(4, 21), datetime.time(19, 27))),
             ('home', (datetime.time(4, 28), datetime.time(19, 28)))]

    def test_to_grid_locator(self):
        expect(sorted(self.locs.to_grid_locator('extsquare'))) == \
            [('Carol', 'JO02ae40'), ('Kenny', 'JO02hu85'), ('home', 'IO92va33')]
        expect(sorted(self.locs.to_grid_locator('subsquare'))) == \
            [('Carol', 'JO02ae'), ('Kenny', 'JO02hu'), ('home', 'IO92va')]



#
# coding=utf-8
"""test___init__ - Test package base"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime

from expecter import expect

from upoints import point


def test_base():
    home = point.Point(52.015, -0.221)
    telford = point.Point(52.6333, -2.5000)
    expect(int(home.distance(telford))) == 169
    expect(int(home.bearing(telford))) == 294
    expect(int(home.final_bearing(telford))) == 293

    expect(home.sun_events(datetime.date(2007, 6, 28))) == \
        (datetime.time(3, 42), datetime.time(20, 24))
    expect(home.sunrise(datetime.date(2007, 6, 28))) == \
        datetime.time(3, 42)
    expect(home.sunset(datetime.date(2007, 6, 28))) == \
        datetime.time(20, 24)



#
# coding=utf-8
"""baken - Imports baken data files"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import re

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser

from upoints import (point, utils)


class Baken(point.Point):

    """Class for representing location from baken_ data files.

    .. versionadded:: 0.4.0

    .. _baken: http://www.qsl.net:80/g4klx/

    """

    __slots__ = ('antenna', 'direction', 'frequency', 'height', '_locator',
                 'mode', 'operator', 'power', 'qth')

    def __init__(self, latitude, longitude, antenna=None, direction=None,
                 frequency=None, height=None, locator=None, mode=None,
                 operator=None, power=None, qth=None):
        """Initialise a new ``Baken`` object.

        :param float latitude: Location's latitude
        :param float longitude: Location's longitude
        :param str antenna: Location's antenna type
        :type direction: ``tuple`` of ``int``
        :param direction: Antenna's direction
        :param float frequency: Transmitter's frequency
        :param float height: Antenna's height
        :param str locator: Location's Maidenhead locator string
        :param str mode: Transmitter's mode
        :type operator: ``tuple`` of ``str``
        :param operator: Transmitter's operator
        :param float power: Transmitter's power
        :param str qth: Location's qth
        :raise LookupError: No position data to use

        """
        if not latitude is None:
            super(Baken, self).__init__(latitude, longitude)
        elif not locator is None:
            latitude, longitude = utils.from_grid_locator(locator)
            super(Baken, self).__init__(latitude, longitude)
        else:
            raise LookupError('Unable to instantiate baken object, no '
                              'latitude or locator string')

        self.antenna = antenna
        self.direction = direction
        self.frequency = frequency
        self.height = height
        self._locator = locator
        self.mode = mode
        self.operator = operator
        self.power = power
        self.qth = qth

    @property
    def locator(self):
        return self._locator

    @locator.setter
    def locator(self, value):
        """Update the locator, and trigger a latitude and longitude update.

        :param str value: New Maidenhead locator string

        """
        self._locator = value
        self._latitude, self._longitude = utils.from_grid_locator(value)

    def __str__(self):
        """Pretty printed location string.

        :param str mode: Coordinate formatting system to use
        :rtype: ``str``
        :return: Human readable string representation of ``Baken`` object

        """
        text = super(Baken, self).__format__('dms')
        if self._locator:
            text = '%s (%s)' % (self._locator, text)
        return text


class Bakens(point.KeyedPoints):

    """Class for representing a group of :class:`Baken` objects.

    .. versionadded:: 0.5.1

    """

    def __init__(self, baken_file=None):
        """Initialise a new `Bakens` object."""
        super(Bakens, self).__init__()
        if baken_file:
            self.import_locations(baken_file)

    def import_locations(self, baken_file):
        """Import baken data files.

        ``import_locations()`` returns a dictionary with keys containing the
        section title, and values consisting of a collection :class:`Baken`
        objects.

        It expects data files in the format used by the baken_ amateur radio
        package, which is Windows INI style files such as:

        .. code-block:: ini

            [Abeche, Chad]
            latitude=14.460000
            longitude=20.680000
            height=0.000000

            [GB3BUX]
            frequency=50.000
            locator=IO93BF
            power=25 TX
            antenna=2 x Turnstile
            height=460
            mode=A1A

        The reader uses the :mod:`configparser` module, so should be reasonably
        robust against encodings and such.  The above file processed by
        ``import_locations()`` will return the following ``dict`` object::

            {"Abeche, Chad": Baken(14.460, 20.680, None, None, None, 0.000,
                                   None, None, None, None, None),
             "GB3BUX": : Baken(None, None, "2 x Turnstile", None, 50.000,
                               460.000, "IO93BF", "A1A", None, 25, None)}

        :type baken_file: ``file``, ``list`` or ``str``
        :param baken_file: Baken data to read
        :rtype: ``dict``
        :return: Named locations and their associated values

        .. _baken: http://www.qsl.net:80/g4klx/

        """
        self._baken_file = baken_file
        data = ConfigParser()
        if hasattr(baken_file, 'readlines'):
            data.readfp(baken_file)
        elif isinstance(baken_file, list):
            data.read(baken_file)
        elif isinstance(baken_file, basestring):
            data.readfp(open(baken_file))
        else:
            raise TypeError('Unable to handle data of type %r'
                            % type(baken_file))
        valid_locator = re.compile(r"[A-Z]{2}\d{2}[A-Z]{2}")
        for name in data.sections():
            elements = {}
            for item in ('latitude', 'longitude', 'antenna', 'direction',
                         'frequency', 'height', 'locator', 'mode', 'operator',
                         'power', 'qth'):
                if data.has_option(name, item):
                    if item in ('antenna', 'locator', 'mode', 'power', 'qth'):
                        elements[item] = data.get(name, item)
                    elif item == 'operator':
                        elements[item] = elements[item].split(',')
                    elif item == 'direction':
                        elements[item] = data.get(name, item).split(',')
                    else:
                        try:
                            elements[item] = data.getfloat(name, item)
                        except ValueError:
                            logging.debug('Multiple frequency workaround for '
                                          '%r entry' % name)
                            elements[item] = \
                                map(float, data.get(name, item).split(','))
                else:
                    elements[item] = None
            if elements['latitude'] is None \
               and not valid_locator.match(elements['locator']):
                logging.info('Skipping %r entry, as it contains no location '
                             'data' % name)
                continue

            self[name] = Baken(**elements)



#
# coding=utf-8
"""test_xearth - Test xearth support"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from unittest import TestCase

from expecter import expect

from upoints.xearth import (Xearth, Xearths)


class TestXearth(TestCase):
    def test___repr__(self):
        expect(repr(Xearth(52.015, -0.221, "James Rowe's house"))) == \
            """Xearth(52.015, -0.221, "James Rowe's house")"""

    def test___str__(self):
        expect(str(Xearth(52.015, -0.221))) == 'N52.015°; W000.221°'
        expect(str(Xearth(52.015, -0.221, "James Rowe's house"))) == \
            "James Rowe's house (N52.015°; W000.221°)"

    def test___format__(self):
        expect(format(Xearth(52.015, -0.221), 'dms')) == \
            """52°00'54"N, 000°13'15"W"""
        expect(format(Xearth(52.015, -0.221), 'dm')) == \
            "52°00.90'N, 000°13.26'W"


class TestXearths(TestCase):
    def test___str__(self):
        markers = Xearths(open('tests/data/xearth'))
        expect(markers.__str__().splitlines()) == \
            ['52.015000 -0.221000 "Home"', '52.633300 -2.500000 "Telford"']

    def test_import_locations(self):
        markers = Xearths(open('tests/data/xearth'))
        expect(str(markers['Home'])) == \
            "James Rowe's home (N52.015°; W000.221°)"
        expect(str(markers['Telford'])) == 'N52.633°; W002.500°'



#
# coding=utf-8
"""test_geonames - Test geonames support"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime

from unittest import TestCase

from expecter import expect

from upoints.geonames import (Location, Locations)
from upoints.utils import FileFormatError


class TestLocation(TestCase):
    def setUp(self):
        self.x = Location(2636782, 'Stotfold', 'Stotfold', None, 52.0,
                          -0.2166667, 'P', 'PPL', 'GB', None, 'F8', None, None,
                          None, 6245, None, 77, 'Europe/London',
                          datetime.date(2007, 6, 15), 0)

    def test___repr__(self):
        expect(repr(self.x)) == \
            ("Location(2636782, 'Stotfold', 'Stotfold', None, 52.0, "
             "-0.2166667, 'P', 'PPL', 'GB', None, 'F8', None, None, None, "
             "6245, None, 77, 'Europe/London', datetime.date(2007, 6, 15), 0)")

    def test___str__(self):
        expect(str(self.x)) == 'Stotfold (N52.000°; W000.217°)'
        self.x.alt_names = ['Home', 'Target']
        expect(str(self.x)) == 'Stotfold (Home, Target - N52.000°; W000.217°)'
        self.x.alt_names = None

    def test___format__(self):
        expect(format(self.x, 'dms')) == \
            """Stotfold (52°00'00"N, 000°13'00"W)"""
        expect(format(self.x, 'dm')) == \
            "Stotfold (52°00.00'N, 000°13.00'W)"


class TestLocations(TestCase):
    def test_import_locations(self):
        locations = Locations(open('tests/data/geonames'))
        expect(str(locations[0])) == \
            'Afon Wyre (River Wayrai, River Wyrai, Wyre - N52.317°; W004.167°)'
        expect(str(locations[1])) == \
            'Wyre (Viera - N59.117°; W002.967°)'
        expect(str(locations[2])) == \
            'Wraysbury (Wyrardisbury - N51.450°; W000.550°)'

        with expect.raises(FileFormatError,
                           "Incorrect data format, if you're using a file "
                           'downloaded from geonames.org please report this '
                           'to James Rowe <jnrowe@gmail.com>'):
            Locations(open('tests/data/broken_geonames'))

    def test_import_timezones_file(self):
        locations = Locations(None, open('tests/data/geonames_timezones'))
        timezones = locations.timezones
        expect(timezones['Asia/Dubai']) == [240, 240]
        expect(timezones['Asia/Kabul']) == [270, 270]
        expect(timezones['Europe/Andorra']) == [60, 120]

        header_skip_check = Locations(None,
                                      open('tests/data/geonames_timezones_header'))
        expect(header_skip_check) == Locations()

        with expect.raises(FileFormatError,
                           "Incorrect data format, if you're using a file "
                           'downloaded from geonames.org please report this '
                           'to James Rowe <jnrowe@gmail.com>'):
            Locations(None, open('tests/data/geonames_timezones_broken'))



#
# coding=utf-8
"""test_gpx - Test gpx support"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from unittest import TestCase

from expecter import expect

from upoints.gpx import (_GpxElem, _GpxMeta, Routepoint, Routepoints,
                         Trackpoint, Trackpoints, Waypoint, Waypoints, etree)
from upoints import point
from upoints import utils

from tests.utils import xml_compare


class Test_GpxElem(TestCase):
    def test___repr__(self):
        expect(repr(_GpxElem(52, 0))) == \
            '_GpxElem(52.0, 0.0, None, None, None, None)'
        expect(repr(_GpxElem(52, 0, None))) == \
            '_GpxElem(52.0, 0.0, None, None, None, None)'
        expect(_GpxElem(52, 0, 'name', 'desc')) == \
            "_GpxElem(52.0, 0.0, 'name', 'desc', None, None)"

    def test___str__(self):
        expect(str(_GpxElem(52, 0))) == """52°00'00"N, 000°00'00"E"""
        expect(str(_GpxElem(52, 0, 'name', 'desc', 40))) == \
            """name (52°00'00"N, 000°00'00"E @ 40m) [desc]"""
        expect(str(_GpxElem(52, 0, 'name', 'desc', 40,
                   utils.Timestamp(2008, 7, 25)))) == \
            ("""name (52°00'00"N, 000°00'00"E @ 40m on """
             '2008-07-25T00:00:00+00:00) [desc]')


class Test_GpxMeta(TestCase):
    def test_togpx(self):
        meta = _GpxMeta(time=(2008, 6, 3, 16, 12, 43, 1, 155, 0))
        expect(etree.tostring(meta.togpx())) == \
            b'<gpx:metadata xmlns:gpx="http://www.topografix.com/GPX/1/1">' \
            b'<gpx:time>2008-06-03T16:12:43+0000</gpx:time>' \
            b'</gpx:metadata>'
        meta.bounds = {'minlat': 52, 'maxlat': 54, 'minlon': -2, 'maxlon': 1}
        expect(etree.tostring(meta.togpx())) == \
            b'<gpx:metadata xmlns:gpx="http://www.topografix.com/GPX/1/1">' \
            b'<gpx:time>2008-06-03T16:12:43+0000</gpx:time><gpx:bounds maxlat="54" maxlon="1" minlat="52" minlon="-2"/>' \
            b'</gpx:metadata>'
        meta.bounds = [point.Point(52.015, -0.221), point.Point(52.167, 0.390)]
        expect(etree.tostring(meta.togpx())) == \
            b'<gpx:metadata xmlns:gpx="http://www.topografix.com/GPX/1/1">' \
            b'<gpx:time>2008-06-03T16:12:43+0000</gpx:time><gpx:bounds maxlat="52.167" maxlon="0.39" minlat="52.015" minlon="-0.221"/>' \
            b'</gpx:metadata>'


class TestWaypoint(TestCase):
    expect(repr(Waypoint(52, 0))) == \
        'Waypoint(52.0, 0.0, None, None, None, None)'
    expect(repr(Waypoint(52, 0, None))) == \
        'Waypoint(52.0, 0.0, None, None, None, None)'
    expect(repr(Waypoint(52, 0, 'name', 'desc'))) == \
        "Waypoint(52.0, 0.0, 'name', 'desc', None, None)"


class TestWaypoints(TestCase):
    def test_import_locations(self):
        waypoints = Waypoints(open('tests/data/gpx'))
        data = list(map(str, sorted(waypoints, key=lambda x: x.name)))
        expect(data[0]) == \
            """Home (52°00'54"N, 000°13'15"W on 2008-07-26T00:00:00+00:00) [My place]"""
        expect(data[1]) == \
            """MSR (52°10'01"N, 000°23'24"E on 2008-07-27T00:00:00+00:00) [Microsoft Research, Cambridge]"""

    def test_export_gpx_file(self):
        locations = Waypoints(open('tests/data/gpx'))
        export = locations.export_gpx_file()
        gpx_xml = etree.parse('tests/data/gpx')
        for e1, e2 in zip(export.getiterator(), gpx_xml.getiterator()):
            xml_compare(e1, e2)



class TestTrackpoint(TestCase):
    def test___repr__(self):
        expect(Trackpoint(52, 0)) == \
            'Trackpoint(52.0, 0.0, None, None, None, None)'
        expect(Trackpoint(52, 0, None)) == \
            'Trackpoint(52.0, 0.0, None, None, None, None)'
        expect(Trackpoint(52, 0, 'name', 'desc')) == \
            "Trackpoint(52.0, 0.0, 'name', 'desc', None, None)"


class TestTrackpoints(TestCase):
    def test_import_locations(self):
        trackpoints = Trackpoints(open('tests/data/gpx_tracks'))
        data = list(map(str, sorted(trackpoints[0], key=lambda x: x.name)))
        expect(data[0]) == \
            """Home (52°00'54"N, 000°13'15"W on 2008-07-26T00:00:00+00:00) [My place]"""
        expect(data[1]) == \
            """MSR (52°10'01"N, 000°23'24"E on 2008-07-27T00:00:00+00:00) [Microsoft Research, Cambridge]"""

    def test_export_gpx_file(self):
        locations = Trackpoints(open('tests/data/gpx_tracks'))
        export = locations.export_gpx_file()
        tracks_xml = etree.parse('tests/data/gpx_tracks')
        for e1, e2 in zip(export.getiterator(), tracks_xml.getiterator()):
            xml_compare(e1, e2)



class TestRoutepoint(TestCase):
    def test___repr__(self):
        expect(Routepoint(52, 0)) == \
            'Routepoint(52.0, 0.0, None, None, None, None)'
        expect(Routepoint(52, 0, None)) == \
            'Routepoint(52.0, 0.0, None, None, None, None)'
        expect(Routepoint(52, 0, 'name', 'desc')) == \
            "Routepoint(52.0, 0.0, 'name', 'desc', None, None)"


class TestRoutepoints(TestCase):
    def test_import_locations(self):
        routepoints = Routepoints(open('tests/data/gpx_routes'))
        data = list(map(str, sorted(routepoints[0], key=lambda x: x.name)))
        expect(data[0]) == \
            """Home (52°00'54"N, 000°13'15"W on 2008-07-26T00:00:00+00:00) [My place]"""
        expect(data[1]) == \
            """MSR (52°10'01"N, 000°23'24"E on 2008-07-27T00:00:00+00:00) [Microsoft Research, Cambridge]"""

    def test_export_gpx_file(self):
        locations = Routepoints(open('tests/data/gpx_routes'))
        export = locations.export_gpx_file()
        routes_xml = etree.parse('tests/data/gpx_routes')

        for e1, e2 in zip(routes_xml.getiterator(), export.getiterator()):
            xml_compare(e1, e2, ellipsis=True)



#
# coding=utf-8
"""xearth - Imports xearth-style marker files"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

__doc__ += """.

.. moduleauthor:: James Rowe <jnrowe@gmail.com>
.. versionadded:: 0.2.0
"""

from upoints import (point, utils)


class Xearth(point.Point):

    """Class for representing a location from a Xearth marker.

    .. versionadded:: 0.2.0

    """

    __slots__ = ('comment', )

    def __init__(self, latitude, longitude, comment=None):
        """Initialise a new ``Xearth`` object.

        :param float latitude: Location's latitude
        :param float longitude: Location's longitude
        :param str comment: Comment for location

        """
        super(Xearth, self).__init__(latitude, longitude)
        self.comment = comment

    def __str__(self):
        """Pretty printed location string.

        .. seealso:

           :class:`point.Point`

        :rtype: ``str``
        :return: Human readable string representation of ``Xearth`` object

        """
        text = super(Xearth, self).__str__()

        if self.comment:
            return '%s (%s)' % (self.comment, text)
        else:
            return text


class Xearths(point.KeyedPoints):

    """Class for representing a group of :class:`Xearth` objects.

    .. versionadded:: 0.5.1

    """

    def __init__(self, marker_file=None):
        """Initialise a new ``Xearths`` object."""
        super(Xearths, self).__init__()
        self._marker_file = marker_file
        if marker_file:
            self.import_locations(marker_file)

    def __str__(self):
        """``Xearth`` objects rendered for use with Xearth/Xplanet.

        :rtype: ``str``
        :return: Xearth/Xplanet marker file formatted output

        """
        return '\n'.join(utils.dump_xearth_markers(self, 'comment'))

    def import_locations(self, marker_file):
        """Parse Xearth data files.

        ``import_locations()`` returns a dictionary with keys containing the
        xearth_ name, and values consisting of a :class:`Xearth` object and
        a string containing any comment found in the marker file.

        It expects Xearth marker files in the following format::

            # Comment

            52.015     -0.221 "Home"          # James Rowe's home
            52.6333    -2.5   "Telford"

        Any empty line or line starting with a '#' is ignored.  All data lines
        are whitespace-normalised, so actual layout should have no effect.  The
        above file processed by ``import_locations()`` will return the
        following ``dict`` object::

            {'Home': point.Point(52.015, -0.221, "James Rowe's home"),
             'Telford': point.Point(52.6333, -2.5, None)}

        .. note:
           This function also handles the extended xplanet_ marker files whose
           points can optionally contain added xplanet specific keywords for
           defining colours and fonts.

        :type marker_file: ``file``, ``list`` or ``str``
        :param marker_file: Xearth marker data to read
        :rtype: ``dict``
        :return: Named locations with optional comments

        .. _xearth: http://hewgill.com/xearth/original/
        .. _xplanet: http://xplanet.sourceforge.net/

        """
        self._marker_file = marker_file
        data = utils.prepare_read(marker_file)

        for line in data:
            line = line.strip()
            if not line or line.startswith('#'):
                continue
            chunk = line.split('#')
            data = chunk[0]
            comment = chunk[1].strip() if len(chunk) == 2 else None
            # Need maximum split of 2, because name may contain whitespace
            latitude, longitude, name = data.split(None, 2)
            name = name.strip()
            # Find matching start and end quote, and keep only the contents
            name = name[1:name.find(name[0], 1)]
            self[name.strip()] = Xearth(latitude, longitude, comment)



#
# coding=utf-8
"""osm - Imports OpenStreetMap data files"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import urllib

from operator import attrgetter

from lxml import etree

from upoints import (point, utils)
from upoints._version import web as ua_string
from upoints.compat import mangle_repr_type

create_elem = utils.element_creator()


def _parse_flags(element):
    """Parse OSM XML element for generic data.

    :param etree.Element element: Element to parse
    :rtype: ``tuple``
    :return: Generic OSM data for object instantiation

    """
    visible = True if element.get('visible') else False
    user = element.get('user')
    timestamp = element.get('timestamp')
    if timestamp:
        timestamp = utils.Timestamp.parse_isoformat(timestamp)
    tags = {}
    try:
        for tag in element['tag']:
            key = tag.get('k')
            value = tag.get('v')
            tags[key] = value
    except AttributeError:
        pass

    return visible, user, timestamp, tags


def _get_flags(osm_obj):
    """Create element independent flags output.

    :param Node osm_obj: Object with OSM-style metadata
    :rtype: ``list``
    :return: Human readable flags output

    """
    flags = []
    if osm_obj.visible:
        flags.append('visible')
    if osm_obj.user:
        flags.append('user: %s' % osm_obj.user)
    if osm_obj.timestamp:
        flags.append('timestamp: %s' % osm_obj.timestamp.isoformat())
    if osm_obj.tags:
        flags.append(', '.join('%s: %s' % (k, v)
                               for k, v in sorted(osm_obj.tags.items())))
    return flags


def get_area_url(location, distance):
    """Generate URL for downloading OSM data within a region.

    This function defines a boundary box where the edges touch a circle of
    ``distance`` kilometres in radius.  It is important to note that the box is
    neither a square, nor bounded within the circle.

    The bounding box is strictly a trapezoid whose north and south edges are
    different lengths, which is longer is dependant on whether the box is
    calculated for a location in the Northern or Southern hemisphere.  You will
    get a shorter north edge in the Northern hemisphere, and vice versa.  This
    is simply because we are applying a flat transformation to a spherical
    object, however for all general cases the difference will be negligible.

    :param Point location: Centre of the region
    :param int distance: Boundary distance in kilometres
    :rtype: ``str``
    :return: URL that can be used to fetch the OSM data within ``distance`` of
        ``location``

    """
    locations = [location.destination(i, distance) for i in range(0, 360, 90)]
    latitudes = list(map(attrgetter('latitude'), locations))
    longitudes = list(map(attrgetter('longitude'), locations))

    bounds = (min(longitudes), min(latitudes), max(longitudes), max(latitudes))

    return ('http://api.openstreetmap.org/api/0.5/map?bbox='
            + ','.join(map(str, bounds)))


class Node(point.Point):

    """Class for representing a node element from OSM data files.

    .. versionadded:: 0.9.0

    """

    __slots__ = ('ident', 'visible', 'user', 'timestamp', 'tags')

    def __init__(self, ident, latitude, longitude, visible=False, user=None,
                 timestamp=None, tags=None):
        """Initialise a new ``Node`` object.

        :param int ident: Unique identifier for the node
        :param float latitude: Nodes's latitude
        :param float longitude: Node's longitude
        :param bool visible: Whether the node is visible
        :param str user: User who logged the node
        :param str timestamp: The date and time a node was logged
        :param dict tags: Tags associated with the node

        """
        super(Node, self).__init__(latitude, longitude)

        self.ident = ident
        self.visible = visible
        self.user = user
        self.timestamp = timestamp
        self.tags = tags

    def __str__(self):
        """Pretty printed location string.

        :rtype: ``str``
        :return: Human readable string representation of ``Node`` object

        """
        text = ['Node %i (%s)' % (self.ident,
                                  super(Node, self).__format__('dms')), ]
        flags = _get_flags(self)

        if flags:
            text.append('[%s]' % ', '.join(flags))
        return ' '.join(text)

    def toosm(self):
        """Generate a OSM node element subtree.

        :rtype: :class:`etree.Element`
        :return: OSM node element

        """
        node = create_elem('node', {'id': str(self.ident),
                                    'lat': str(self.latitude),
                                    'lon': str(self.longitude)})
        node.set('visible', 'true' if self.visible else 'false')
        if self.user:
            node.set('user', self.user)
        if self.timestamp:
            node.set('timestamp', self.timestamp.isoformat())
        if self.tags:
            for key, value in sorted(self.tags.items()):
                node.append(create_elem('tag', {'k': key, 'v': value}))

        return node

    def get_area_url(self, distance):
        """Generate URL for downloading OSM data within a region.

        :param int distance: Boundary distance in kilometres
        :rtype: ``str``
        :return: URL that can be used to fetch the OSM data within ``distance``
            of ``location``

        """
        return get_area_url(self, distance)

    def fetch_area_osm(self, distance):
        """Fetch, and import, an OSM region.

        :param int distance: Boundary distance in kilometres
        :rtype: :class:`Osm`
        :return: All the data OSM has on a region imported for use

        """
        return Osm(urllib.urlopen(get_area_url(self, distance)))

    @staticmethod
    def parse_elem(element):
        """Parse a OSM node XML element.

        :param etree.Element element: XML Element to parse
        :rtype: ``Node``
        :return: ``Node`` object representing parsed element

        """
        ident = int(element.get('id'))
        latitude = element.get('lat')
        longitude = element.get('lon')

        flags = _parse_flags(element)

        return Node(ident, latitude, longitude, *flags)


@mangle_repr_type
class Way(point.Points):

    """Class for representing a way element from OSM data files.

    .. versionadded:: 0.9.0

    """

    __slots__ = ('ident', 'visible', 'user', 'timestamp', 'tags')

    def __init__(self, ident, nodes, visible=False, user=None, timestamp=None,
                 tags=None):
        """Initialise a new ``Way`` object.

        :param int ident: Unique identifier for the way
        :type nodes: ``list`` of ``str`` objects
        :param nodes: Identifiers of the nodes that form this way
        :param bool visible: Whether the way is visible
        :param str user: User who logged the way
        :param str timestamp: The date and time a way was logged
        :param dict tags: Tags associated with the way

        """
        super(Way, self).__init__()

        self.extend(nodes)

        self.ident = ident
        self.visible = visible
        self.user = user
        self.timestamp = timestamp
        self.tags = tags

    def __repr__(self):
        """Self-documenting string representation.

        :rtype: ``str``
        :return: String to recreate ``Way`` object

        """
        return utils.repr_assist(self, {'nodes': self[:]})

    def __str__(self, nodes=False):
        """Pretty printed location string.

        :param list nodes: Nodes to be used in expanding references
        :rtype: ``str``
        :return: Human readable string representation of ``Way`` object

        """
        text = ['Way %i' % (self.ident), ]
        if not nodes:
            text.append(' (nodes: %s)' % str(self[:])[1:-1])
        flags = _get_flags(self)

        if flags:
            text.append(' [%s]' % ', '.join(flags))
        if nodes:
            text.append('\n')
            text.append('\n'.join('    %s' % nodes[node] for node in self[:]))

        return ''.join(text)

    def toosm(self):
        """Generate a OSM way element subtree.

        :rtype: :class:`etree.Element`
        :return: OSM way element

        """
        way = create_elem('way', {'id': str(self.ident)})
        way.set('visible', 'true' if self.visible else 'false')
        if self.user:
            way.set('user', self.user)
        if self.timestamp:
            way.set('timestamp', self.timestamp.isoformat())
        if self.tags:
            for key, value in sorted(self.tags.items()):
                way.append(create_elem('tag', {'k': key, 'v': value}))

        for node in self:
            way.append(create_elem('nd', {'ref': str(node)}))

        return way

    @staticmethod
    def parse_elem(element):
        """Parse a OSM way XML element.

        :param etree.Element element: XML Element to parse
        :rtype: ``Way``
        :return: `Way` object representing parsed element

        """
        ident = int(element.get('id'))
        flags = _parse_flags(element)
        nodes = [node.get('ref') for node in element.findall('nd')]
        return Way(ident, nodes, *flags)


class Osm(point.Points):

    """Class for representing an OSM region.

    .. versionadded:: 0.9.0

    """

    def __init__(self, osm_file=None):
        """Initialise a new ``Osm`` object."""
        super(Osm, self).__init__()
        self._osm_file = osm_file
        if osm_file:
            self.import_locations(osm_file)
        self.generator = ua_string
        self.version = '0.5'

    def import_locations(self, osm_file):
        """Import OSM data files.

        ``import_locations()`` returns a list of ``Node`` and ``Way`` objects.

        It expects data files conforming to the `OpenStreetMap 0.5 DTD`_, which
        is XML such as::

            <?xml version="1.0" encoding="UTF-8"?>
            <osm version="0.5" generator="upoints/0.9.0">
              <node id="0" lat="52.015749" lon="-0.221765" user="jnrowe" visible="true" timestamp="2008-01-25T12:52:11+00:00" />
              <node id="1" lat="52.015761" lon="-0.221767" visible="true" timestamp="2008-01-25T12:53:00+00:00">
                <tag k="created_by" v="hand" />
                <tag k="highway" v="crossing" />
              </node>
              <node id="2" lat="52.015754" lon="-0.221766" user="jnrowe" visible="true" timestamp="2008-01-25T12:52:30+00:00">
                <tag k="amenity" v="pub" />
              </node>
              <way id="0" visible="true" timestamp="2008-01-25T13:00:00+0000">
                <nd ref="0" />
                <nd ref="1" />
                <nd ref="2" />
                <tag k="ref" v="My Way" />
                <tag k="highway" v="primary" />
              </way>
            </osm>

        The reader uses the :mod:`ElementTree` module, so should be very fast
        when importing data.  The above file processed by
        ``import_locations()`` will return the following `Osm` object::

            Osm([
                Node(0, 52.015749, -0.221765, True, "jnrowe",
                     utils.Timestamp(2008, 1, 25, 12, 52, 11), None),
                Node(1, 52.015761, -0.221767, True,
                     utils.Timestamp(2008, 1, 25, 12, 53), None,
                     {"created_by": "hand", "highway": "crossing"}),
                Node(2, 52.015754, -0.221766, True, "jnrowe",
                     utils.Timestamp(2008, 1, 25, 12, 52, 30),
                     {"amenity": "pub"}),
                Way(0, [0, 1, 2], True, None,
                    utils.Timestamp(2008, 1, 25, 13, 00),
                    {"ref": "My Way", "highway": "primary"})],
                generator="upoints/0.9.0")

        :type osm_file: ``file``, ``list`` or ``str``
        :param osm_file: OpenStreetMap data to read
        :rtype: ``Osm``
        :return: Nodes and ways from the data

        .. _OpenStreetMap 0.5 DTD:
            http://wiki.openstreetmap.org/wiki/OSM_Protocol_Version_0.5/DTD

        """
        self._osm_file = osm_file
        data = utils.prepare_xml_read(osm_file, objectify=True)

        # This would be a lot simpler if OSM exports defined a namespace
        if not data.tag == 'osm':
            raise ValueError("Root element %r is not `osm'" % data.tag)
        self.version = data.get('version')
        if not self.version:
            raise ValueError('No specified OSM version')
        elif not self.version == '0.5':
            raise ValueError('Unsupported OSM version %r' % data)

        self.generator = data.get('generator')

        for elem in data.getchildren():
            if elem.tag == 'node':
                self.append(Node.parse_elem(elem))
            elif elem.tag == 'way':
                self.append(Way.parse_elem(elem))

    def export_osm_file(self):
        """Generate OpenStreetMap element tree from `Osm`"""
        osm = create_elem('osm', {'generator': self.generator,
                                  'version': self.version})
        osm.extend(obj.toosm() for obj in self)

        return etree.ElementTree(osm)



#
# coding=utf-8
"""tzdata - Imports timezone data files from UNIX zoneinfo"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from operator import attrgetter

from upoints import (point, utils)
from upoints.compat import mangle_repr_type


@mangle_repr_type
class Zone(point.Point):

    """Class for representing timezone descriptions from zoneinfo data.

    .. versionadded:: 0.6.0

    """

    __slots__ = ('country', 'zone', 'comments')

    def __init__(self, location, country, zone, comments=None):
        """Initialise a new ``Zone`` object.

        :param str location: Primary location in ISO 6709 format
        :param str country: Location's ISO 3166 country code
        :param str zone: Location's zone name as used in zoneinfo database
        :param list comments: Location's alternate names

        """
        latitude, longitude = utils.from_iso6709(location + '/')[:2]
        super(Zone, self).__init__(latitude, longitude)

        self.country = country
        self.zone = zone
        self.comments = comments

    def __repr__(self):
        """Self-documenting string representation.

        :rtype: ``str``
        :return: String to recreate ``Zone`` object

        """
        location = utils.to_iso6709(self.latitude, self.longitude,
                                    format='dms')[:-1]
        return utils.repr_assist(self, {'location': location})

    def __str__(self):
        """Pretty printed location string.

        :param str mode: Coordinate formatting system to use
        :rtype: ``str``
        :return: Human readable string representation of ``Zone`` object

        """
        text = ['%s (%s: %s' % (self.zone, self.country,
                                super(Zone, self).__format__('dms')), ]
        if self.comments:
            text.append(' also ' + ', '.join(self.comments))
        text.append(')')
        return ''.join(text)


class Zones(point.Points):

    """Class for representing a group of :class:`Zone` objects.

    .. versionadded:: 0.6.0

    """

    def __init__(self, zone_file=None):
        """Initialise a new Zones object."""
        super(Zones, self).__init__()
        self._zone_file = zone_file
        if zone_file:
            self.import_locations(zone_file)

    def import_locations(self, zone_file):
        """Parse zoneinfo zone description data files.

        ``import_locations()`` returns a list of :class:`Zone` objects.

        It expects data files in one of the following formats::

            AN	+1211-06900	America/Curacao
            AO	-0848+01314	Africa/Luanda
            AQ	-7750+16636	Antarctica/McMurdo	McMurdo Station, Ross Island

        Files containing the data in this format can be found in the
        :file:`zone.tab` file that is normally found in
        :file:`/usr/share/zoneinfo` on UNIX-like systems, or from the `standard
        distribution site`_.

        When processed by ``import_locations()`` a ``list`` object of the
        following style will be returned::

            [Zone(None, None, "AN", "America/Curacao", None),
             Zone(None, None, "AO", "Africa/Luanda", None),
             Zone(None, None, "AO", "Antartica/McMurdo",
                  ["McMurdo Station", "Ross Island"])]

        :type zone_file: ``file``, ``list`` or ``str``
        :param zone_file: ``zone.tab`` data to read
        :rtype: ``list``
        :return: Locations as :class:`Zone` objects
        :raise FileFormatError: Unknown file format

        .. _standard distribution site: ftp://elsie.nci.nih.gov/pub/

        """
        self._zone_file = zone_file
        field_names = ('country', 'location', 'zone', 'comments')

        data = utils.prepare_csv_read(zone_file, field_names, delimiter=r"	")

        for row in (x for x in data if not x['country'].startswith('#')):
            if row['comments']:
                row['comments'] = row['comments'].split(', ')
            self.append(Zone(**row))

    def dump_zone_file(self):
        """Generate a zoneinfo compatible zone description table.

        :rtype: ``list``
        :return: zoneinfo descriptions

        """
        data = []
        for zone in sorted(self, key=attrgetter('country')):
            text = ['%s	%s	%s'
                    % (zone.country,
                       utils.to_iso6709(zone.latitude, zone.longitude,
                                        format='dms')[:-1],
                       zone.zone), ]
            if zone.comments:
                text.append('	%s' % ', '.join(zone.comments))
            data.append(''.join(text))
        return data



#
# coding=utf-8
"""kml - Imports KML data files"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from lxml import etree

from upoints import (point, trigpoints, utils)

KML_NS = 'http://earth.google.com/kml/2.2'
etree.register_namespace('kml', KML_NS)

create_elem = utils.element_creator(KML_NS)


class Placemark(trigpoints.Trigpoint):

    """Class for representing a Placemark element from KML data files.

    .. versionadded:: 0.6.0

    """

    __slots__ = ('description', )

    def __init__(self, latitude, longitude, altitude=None, name=None,
                 description=None):
        """Initialise a new ``Placemark`` object.

        :param float latitude: Placemarks's latitude
        :param float longitude: Placemark's longitude
        :param float altitude: Placemark's altitude
        :param str name: Name for placemark
        :param str description: Placemark's description

        """
        super(Placemark, self).__init__(latitude, longitude, altitude, name)

        if altitude:
            self.altitude = float(altitude)
        self.description = description

    def __str__(self):
        """Pretty printed location string.

        :rtype: ``str``
        :return: Human readable string representation of ``Placemark`` object

        """
        location = super(Placemark, self).__format__('dms')
        if self.description:
            return '%s [%s]' % (location, self.description)
        else:
            return location

    def tokml(self):
        """Generate a KML Placemark element subtree.

        :rtype: :class:`etree.Element`
        :return: KML Placemark element

        """
        placemark = create_elem('Placemark')
        if self.name:
            placemark.set('id', self.name)
            placemark.name = create_elem('name', text=self.name)
        if self.description:
            placemark.description = create_elem('description',
                                                text=self.description)
        placemark.Point = create_elem('Point')

        data = [str(self.longitude), str(self.latitude)]
        if self.altitude:
            if int(self.altitude) == self.altitude:
                data.append('%i' % self.altitude)
            else:
                data.append(str(self.altitude))
        placemark.Point.coordinates = create_elem('coordinates',
                                                  text=','.join(data))

        return placemark


class Placemarks(point.KeyedPoints):

    """Class for representing a group of :class:`Placemark` objects.

    .. versionadded:: 0.6.0

    """

    def __init__(self, kml_file=None):
        """Initialise a new ``Placemarks`` object."""
        super(Placemarks, self).__init__()
        self._kml_file = kml_file
        if kml_file:
            self.import_locations(kml_file)

    def import_locations(self, kml_file):
        """Import KML data files.

        ``import_locations()`` returns a dictionary with keys containing the
        section title, and values consisting of :class:`Placemark` objects.

        It expects data files in KML format, as specified in `KML Reference`_,
        which is XML such as::

            <?xml version="1.0" encoding="utf-8"?>
            <kml xmlns="http://earth.google.com/kml/2.1">
                <Document>
                    <Placemark id="Home">
                        <name>Home</name>
                        <Point>
                            <coordinates>-0.221,52.015,60</coordinates>
                        </Point>
                    </Placemark>
                    <Placemark id="Cambridge">
                        <name>Cambridge</name>
                        <Point>
                            <coordinates>0.390,52.167</coordinates>
                        </Point>
                    </Placemark>
                </Document>
            </kml>

        The reader uses the :mod:`ElementTree` module, so should be very fast
        when importing data.  The above file processed by
        ``import_locations()`` will return the following ``dict`` object::

            {"Home": Placemark(52.015, -0.221, 60),
             "Cambridge": Placemark(52.167, 0.390, None)}

        :type kml_file: ``file``, ``list`` or ``str``
        :param kml_file: KML data to read
        :rtype: ``dict``
        :return: Named locations with optional comments

        .. _KML Reference:
           http://code.google.com/apis/kml/documentation/kmlreference.html

        """
        self._kml_file = kml_file
        data = utils.prepare_xml_read(kml_file, objectify=True)

        for place in data.Document.Placemark:
            name = place.name.text
            coords = place.Point.coordinates.text
            if coords is None:
                logging.info('No coordinates found for %r entry' % name)
                continue
            coords = coords.split(',')
            if len(coords) == 2:
                longitude, latitude = coords
                altitude = None
            elif len(coords) == 3:
                longitude, latitude, altitude = coords
            else:
                raise ValueError('Unable to handle coordinates value %r'
                                 % coords)
            try:
                description = place.description
            except AttributeError:
                description = None
            self[name] = Placemark(latitude, longitude, altitude, name,
                                   description)

    def export_kml_file(self):
        """Generate KML element tree from ``Placemarks``.

        :rtype: :class:`etree.ElementTree`
        :return: KML element tree depicting ``Placemarks``

        """
        kml = create_elem('kml')
        kml.Document = create_elem('Document')
        for place in sorted(self.values(), key=lambda x: x.name):
            kml.Document.append(place.tokml())

        return etree.ElementTree(kml)



#
# coding=utf-8
"""conf - Sphinx configuration information"""
# Copyright © 2011-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys

from glob import glob
from subprocess import (CalledProcessError, check_output)

from PIL import Image

root_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
sys.path.insert(0, root_dir)


import upoints

for image in glob('.static/*.png'):
    if '_mini.png' in image:
        continue
    thumb_name = image.replace('.png', '_mini.png')
    if not os.path.exists(thumb_name) \
            or os.path.getmtime(image) > os.path.getmtime(thumb_name):
        print 'Creating thumbnail for', image
        try:
            image = Image.open(image)
            image.thumbnail((256, 192))
            image.save(thumb_name)
        except IOError:
            print 'cannot create thumbnail for', image
            sys.exit(1)

extensions = \
    ['sphinx.ext.%s' % ext for ext in ['autodoc', 'coverage', 'doctest',
                                       'intersphinx', 'viewcode', 'todo']] \
    + ['sphinxcontrib.%s' % ext for ext in []]

# Only activate spelling, if it is installed.  It is not required in the
# general case and we don't have the granularity to describe this in a clean
# way
try:
    from sphinxcontrib import spelling  # NOQA
except ImportError:
    pass
else:
    extensions.append('sphinxcontrib.spelling')

master_doc = 'index'
source_suffix = '.rst'

project = u'upoints'
copyright = upoints.__copyright__

version = '.'.join(map(str, upoints._version.tuple[:2]))
release = upoints._version.dotted

pygments_style = 'sphinx'
html_theme_options = {
    'externalrefs': True,
}
try:
    html_last_updated_fmt = check_output(['git', 'log',
                                          "--pretty=format:'%ad [%h]'",
                                          '--date=short', '-n1'])
except CalledProcessError:
    pass

man_pages = [
    ('edist.1', 'edist', u'upoints Documentation', [u'James Rowe'], 1)
]

todo_include_todos = True

# Autodoc extension settings
autoclass_content = 'both'
autodoc_default_flags = ['members', ]

intersphinx_mapping = {
    'python': ('http://docs.python.org/', os.getenv('SPHINX_PYTHON_OBJECTS')),
}

spelling_lang = 'en_GB'
spelling_word_list_filename = 'wordlist.txt'



#
# coding=utf-8
"""geonames - Imports geonames.org data files"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime

try:
    from dateutil import tz
except ImportError:
    #: ``dateutil`` module reference if available
    tz = None

from upoints import (point, trigpoints, utils)


class Location(trigpoints.Trigpoint):

    """Class for representing a location from a geonames.org_ data file.

    All country codes are specified with their two letter ISO-3166 country
    code.

    .. versionadded:: 0.3.0

    :cvar __TIMEZONES: ``dateutil.gettz`` cache to speed up generation

    .. _geonames.org: http://www.geonames.org/

    """

    __slots__ = ('geonameid', 'asciiname', 'alt_names', 'feature_class',
                 'feature_code', 'country', 'alt_country', 'admin1', 'admin2',
                 'admin3', 'admin4', 'population', 'altitude', 'gtopo30',
                 'tzname', 'modified_date')

    if tz:
        __TIMEZONES = {}

    def __init__(self, geonameid, name, asciiname, alt_names, latitude,
                 longitude, feature_class, feature_code, country, alt_country,
                 admin1, admin2, admin3, admin4, population, altitude, gtopo30,
                 tzname, modified_date, timezone=None):
        """Initialise a new ``Location`` object.

        :param int geonameid: ID of record in geonames database
        :param unicode name: Name of geographical location
        :param str asciiname: Name of geographical location in ASCII encoding
        :type alt_names: ``list`` of ``unicode``
        :param alt_names: Alternate names for the location
        :param float latitude: Location's latitude
        :param float longitude: Location's longitude
        :param str feature_class: Location's type
        :param str feature_code: Location's code
        :param str country: Location's country
        :param str alt_country: Alternate country codes for location
        :param str admin1: FIPS code (subject to change to ISO code), ISO code
            for the US and CH
        :param str admin2: Code for the second administrative division, a county
            in the US
        :param str admin3: Code for third level administrative division
        :param str admin4: Code for fourth level administrative division
        :param int population: Location's population, if applicable
        :param int altitude: Location's elevation
        :param int gtopo30: Average elevation of 900 square metre region, if
            available
        :param str tzname: The timezone identifier using POSIX timezone names
        :param datetime.date modified_date: Location's last modification date
            in the geonames databases
        :param int timezone: The non-DST timezone offset from UTC in minutes

        """
        super(Location, self).__init__(latitude, longitude, altitude, name)
        self.geonameid = geonameid
        self.name = name
        self.asciiname = asciiname
        self.alt_names = alt_names
        self.latitude = latitude
        self.longitude = longitude
        self.feature_class = feature_class
        self.feature_code = feature_code
        self.country = country
        self.alt_country = alt_country
        self.admin1 = admin1
        self.admin2 = admin2
        self.admin3 = admin3
        self.admin4 = admin4
        self.population = population
        self.altitude = altitude
        self.gtopo30 = gtopo30
        self.tzname = tzname
        self.modified_date = modified_date
        if timezone is not None:
            self.timezone = timezone
        elif tz:
            if tzname in Location.__TIMEZONES:
                self.timezone = Location.__TIMEZONES[tzname]
            else:
                self.timezone = int(tz.gettz(tzname)._ttinfo_std.offset / 60)
                Location.__TIMEZONES[tzname] = self.timezone
        else:
            self.timezone = None

    def __str__(self):
        """Pretty printed location string.

        .. seealso::

           :type :class:`trigpoints.point.Point`

        :rtype: ``str``
        :return: Human readable string representation of ``Location`` object

        """
        return self.__format__()

    def __format__(self, format_spec='dd'):
        """Extended pretty printing for location strings.

        :param str format_spec: Coordinate formatting system to use
        :rtype: ``str``
        :return: Human readable string representation of ``Point`` object
        :raise ValueError: Unknown value for ``format_spec``

        """
        text = super(Location.__base__, self).__format__(format_spec)

        if self.alt_names:
            return '%s (%s - %s)' % (self.name, ', '.join(self.alt_names),
                                     text)
        else:
            return '%s (%s)' % (self.name, text)


class Locations(point.Points):

    """Class for representing a group of :class:`Location` objects.

    .. versionadded:: 0.5.1

    """

    def __init__(self, data=None, tzfile=None):
        """Initialise a new ``Locations`` object."""
        super(Locations, self).__init__()
        if tzfile:
            self.import_timezones_file(tzfile)
        else:
            self.timezones = {}

        self._data = data
        self._tzfile = tzfile

        if data:
            self.import_locations(data)

    def import_locations(self, data):
        """Parse geonames.org country database exports.

        ``import_locations()`` returns a list of :class:`trigpoints.Trigpoint`
        objects generated from the data exported by geonames.org_.

        It expects data files in the following tab separated format::

            2633441	Afon Wyre	Afon Wyre	River Wayrai,River Wyrai,Wyre	52.3166667	-4.1666667	H	STM	GB	GB	00				0		-9999	Europe/London	1994-01-13
            2633442	Wyre	Wyre	Viera	59.1166667	-2.9666667	T	ISL	GB	GB	V9				0		1	Europe/London	2004-09-24
            2633443	Wraysbury	Wraysbury	Wyrardisbury	51.45	-0.55	P	PPL	GB		P9				0		28	Europe/London	2006-08-21

        Files containing the data in this format can be downloaded from the
        geonames.org_ site in their `database export page`_.

        Files downloaded from the geonames site when processed by
        ``import_locations()`` will return ``list`` objects of the following
        style::

            [Location(2633441, "Afon Wyre", "Afon Wyre",
                      ['River Wayrai', 'River Wyrai', 'Wyre'],
                      52.3166667, -4.1666667, "H", "STM", "GB", ['GB'], "00",
                      None, None, None, 0, None, -9999, "Europe/London",
                      datetime.date(1994, 1, 13)),
             Location(2633442, "Wyre", "Wyre", ['Viera'], 59.1166667,
                      -2.9666667, "T", "ISL", "GB", ['GB'], "V9", None, None,
                      None, 0, None, 1, "Europe/London",
                      datetime.date(2004, 9, 24)),
             Location(2633443, "Wraysbury", "Wraysbury", ['Wyrardisbury'],
                      51.45, -0.55, "P", "PPL", "GB", None, "P9", None, None,
                      None, 0, None, 28, "Europe/London",
                      datetime.date(2006, 8, 21))]

        :type data: ``file``, ``list`` or ``str``
        :param data: geonames.org locations data to read
        :rtype: ``list``
        :return: geonames.org identifiers with :class:`Location` objects
        :raise FileFormatError: Unknown file format

        .. _geonames.org: http://www.geonames.org/
        .. _database export page: http://download.geonames.org/export/dump/

        """
        self._data = data
        field_names = ('geonameid', 'name', 'asciiname', 'alt_names',
                       'latitude', 'longitude', 'feature_class',
                       'feature_code', 'country', 'alt_country', 'admin1',
                       'admin2', 'admin3', 'admin4', 'population', 'altitude',
                       'gtopo30', 'tzname', 'modified_date')
        comma_split = lambda s: s.split(',')
        date_parse = lambda s: datetime.date(*map(int, s.split('-')))
        or_none = lambda x, s: x(s) if s else None
        str_or_none = lambda s: or_none(str, s)
        float_or_none = lambda s: or_none(float, s)
        int_or_none = lambda s: or_none(int, s)
        tz_parse = lambda s: self.timezones[s][0] if self.timezones else None
        field_parsers = (int_or_none, str_or_none, str_or_none, comma_split,
                         float_or_none, float_or_none, str_or_none,
                         str_or_none, str_or_none, comma_split, str_or_none,
                         str_or_none, str_or_none, str_or_none, int_or_none,
                         int_or_none, int_or_none, tz_parse, date_parse)
        data = utils.prepare_csv_read(data, field_names, delimiter=r"	")
        for row in data:
            try:
                for name, parser in zip(field_names, field_parsers):
                    row[name] = parser(row[name])
            except ValueError:
                raise utils.FileFormatError('geonames.org')
            self.append(Location(**row))

    def import_timezones_file(self, data):
        """Parse geonames.org_ timezone exports.

        ``import_timezones_file()`` returns a dictionary with keys containing
        the timezone identifier, and values consisting of a UTC offset and UTC
        offset during daylight savings time in minutes.

        It expects data files in the following format::

            Europe/Andorra	1.0	2.0
            Asia/Dubai	4.0	4.0
            Asia/Kabul	4.5	4.5

        Files containing the data in this format can be downloaded from the
        geonames site in their `database export page`_

        Files downloaded from the geonames site when processed by
        ``import_timezones_file()`` will return ``dict`` object of the
        following style::

            {"Europe/Andorra": (60, 120),
             "Asia/Dubai": (240, 240),
             "Asia/Kabul": (270, 270)}

        :type data: ``file``, ``list`` or ``str``
        :param data: geonames.org timezones data to read
        :rtype: ``list``
        :return: geonames.org timezone identifiers with their UTC offsets
        :raise FileFormatError: Unknown file format

        .. _geonames.org: http://www.geonames.org/
        .. _database export page: http://download.geonames.org/export/dump/

        """
        self._tzfile = data
        field_names = ('ident', 'gmt_offset', 'dst_offset')
        time_parse = lambda n: int(float(n) * 60)
        data = utils.prepare_csv_read(data, field_names, delimiter=r"	")

        self.timezones = {}
        for row in data:
            if row['ident'] == 'TimeZoneId':
                continue
            try:
                delta = list(map(time_parse, (row['gmt_offset'], row['dst_offset'])))
            except ValueError:
                raise utils.FileFormatError('geonames.org')
            self.timezones[row['ident']] = delta



#
# coding=utf-8
"""test_weather_stations - Test weather stations support"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from unittest import TestCase

from expecter import expect

from upoints.weather_stations import (Station, Stations)


class TestStation(TestCase):
    def setUp(self):
        self.x = Station('EGLL', 'London / Heathrow Airport', None,
                         'United Kingdom', 6, 51.4833333333, -0.45, None, None,
                         24, 0, True)

    def test___repr__(self):
        expect(repr(self.x)) == \
            ("Station('EGLL', 'London / Heathrow Airport', None, "
             "'United Kingdom', 6, 51.4833333333, -0.45, None, None, 24, 0, "
             'True)')

    def test___str__(self):
        expect(str(self.x)) == \
            'London / Heathrow Airport (EGLL - N51.483°; W000.450°)'
        self.x.alt_id = None
        expect(str(self.x)) == \
            'London / Heathrow Airport (N51.483°; W000.450°)'
        self.x.alt_id = 'EGLL'

    def test___format__(self):
        expect(format(self.x, 'dms')) == \
            """London / Heathrow Airport (EGLL - 51°28'59"N, 000°27'00"W)"""
        expect(format(self.x, 'dm')) == \
            "London / Heathrow Airport (EGLL - 51°29.00'N, 000°27.00'W)"


class TestStations(TestCase):
    def test_import_locations_wmo(self):
        stations = Stations(open('tests/data/WMO_stations'))
        data = sorted(stations.items())
        expect('%s - %s' % data[0]) == \
            '00000 - Buckland, Buckland Airport (PABL - N65.982°; W161.152°)'
        expect('%s - %s' % data[1]) == \
            '01001 - Jan Mayen (ENJA - N70.933°; W008.667°)'
        expect('%s - %s' % data[2]) == \
            '01002 - Grahuken (N79.783°; E014.467°)'

    def test_import_locations_icao(self):
        stations = Stations(open('tests/data/ICAO_stations'), 'ICAO')
        data = sorted(stations.items())
        expect('%s - %s' % data[0]) == \
            'AYMD - Madang (94014 - S05.217°; E145.783°)'
        expect('%s - %s' % data[1]) == \
            'AYMO - Manus Island/Momote (S02.062°; E147.424°)'
        expect('%s - %s' % data[2]) == \
            'AYPY - Moresby (94035 - S09.433°; E147.217°)'

    def test_import_locations_broken_wmo(self):
        stations = Stations(open('tests/data/broken_WMO_stations'))
        data = sorted(stations.items())
        expect('%s - %s' % data[0]) == \
            '71046 - Komakuk Beach, Y. T. (CWKM - N69.617°; W140.200°)'
        expect('%s - %s' % data[1]) == \
            '71899 - Langara, B. C. (CWLA - N54.250°; W133.133°)'

    def test_import_locations_broken_icao(self):
        stations = Stations(open('tests/data/broken_ICAO_stations'), 'ICAO')
        data = sorted(stations.items())
        expect('%s - %s' % data[0]) == \
            'KBRX - Bordeaux (N41.933°; W104.950°)'
        expect('%s - %s' % data[1]) == \
            'KCQB - Chandler, Chandler Municipal Airport (N35.724°; W096.820°)'
        expect('%s - %s' % data[2]) == \
            'KTYR - Tyler, Tyler Pounds Field (N32.359°; W095.404°)'



#
# coding=utf-8
"""mock - Mock objects for doctest code snippets"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

try:
    import __builtin__
except ImportError:
    import builtins as __builtin__
import os
import StringIO
import urllib

from types import ModuleType

from tests import grab_net_sources
SOURCES = dict((os.path.basename(s), s) for s in grab_net_sources.SOURCES)
BASEDIR = os.path.dirname(__file__)

def isfile(path):
    """Mock `isfile` to check existence of test files.

    :Parameters:
        path : `str`
            File to check for existence
    :rtype: `bool`
    :return: `True` if file exists, `False` otherwise

    """
    filename = os.path.basename(path)
    try:
        __builtin__.open(os.path.join(BASEDIR, 'data', filename))
    except IOError:
        return False
    return True

def _get_test_file(filename):
    """Open a test data file.

    :Parameters:
        filename : `str`
            Basename of the test data to open
    :rtype: `file`
    :return: Test data
    :raise IOError: When the file can't be opened for reading

    """
    if isfile(filename):
        return __builtin__.open(os.path.join(BASEDIR, 'data', filename))
    else:
        if filename in SOURCES:
            raise IOError('%r missing.  It can be downloaded from %r, or '
                          "alternatively by running the `grab_net_sources' "
                          'script.' % (filename, SOURCES[filename]))
        else:
            raise IOError('Can not open %r' % filename)

def open(filename, mode='rb'):
    """Mock `open` function to open test data files.

    :Parameters:
        filename : `str`
            File to simulate, basename is used as local file name
        mode : `str`
            Valid `file` mode string
    :rtype: `file`
    :return: File object opened from test data directory, or ``StringIO.StringIO``
        object if a writable file is expected
    :raise NotImplementedError: When attempting to use an unhandled file mode

    """
    if 'r' in mode:
        return _get_test_file(os.path.basename(filename))
    elif 'w' in mode:
        return StringIO.StringIO()
    else:
        raise NotImplementedError

def urlopen(url, data=None, proxies=None):
    """Mock `urlopen` to open test data files.

    :Parameters:
        url : `str`
            URL to simulate, basename is used as local file name
        data : any
            Ignored, just for compatibility with `urlopen` callers
        proxies : any
            Ignored, just for compatibility with `urlopen` callers
    :rtype: `file`
    :return: File object from test data directory

    """
    return _get_test_file(os.path.basename(url))
urllib.urlopen = urlopen

class pymetar(ModuleType):
    """Mock `pymetar` infrastructure for tests.

    :since: 0.6.0

    :see: `pymetar <http://www.schwarzvogel.de/software-pymetar.shtml>`__

    """
    class ReportFetcher(object):
        def __init__(self, StationCode=None):
            """Mock `ReportFetcher` initialisation for tests.

            :Parameters:
                StationCode : any
                    Ignored, just for compatibility with `ReportFetcher`
                    callers

            """
            super(pymetar.ReportFetcher, self).__init__()

        @staticmethod
        def FetchReport():
            """Mock ``FetchReport`` function to pass tests."""
            pass


    class ReportParser(object):
        class ParseReport(object):
            def __init__(self, MetarReport=None):
                """Mock ``ParseReport`` object to return test data.

                :Parameters:
                    MetarReport : any
                        Ignored, just for compatibility with ``FetchReport``
                        callers

                """
                super(pymetar.ReportParser.ParseReport, self).__init__()

            @staticmethod
            def getTemperatureCelsius():
                """Mock `getTemperatureCelsius`

                :rtype: `float`
                :return: Sample temperature data for tests

                """
                return 10.3

            @staticmethod
            def getISOTime():
                """Mock `getISOTime`

                :rtype: `str`
                :return: Sample ISO time string

                """
                return '2007-11-28 19:20:00Z'




#
# coding=utf-8
"""test_osm - Test osm support"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from unittest import TestCase

from expecter import expect

from upoints import (point, utils)
from upoints.osm import (Node, Osm, Way, etree, get_area_url)

from tests.utils import (xml_compare, xml_str_compare)


def test_get_area_url():
    expect(get_area_url(point.Point(52.015, -0.221), 3)) == \
        'http://api.openstreetmap.org/api/0.5/map?bbox=%s,%s,%s,%s' \
        % (-0.26486443825283734, 51.98800340214556, -0.17713556174716266,
           52.04199659785444)

    expect(get_area_url(point.Point(52.015, -0.221), 12)) == \
        'http://api.openstreetmap.org/api/0.5/map?bbox=%s,%s,%s,%s' \
        % (-0.3964574335910109, 51.907013608582226, -0.04554256640898919,
           52.12298639141776)


class TestNode(TestCase):
    def setUp(self):
        self.bare = Node(0, 52, 0)
        self.named = Node(0, 52, 0, True, 'jnrowe',
                          utils.Timestamp(2008, 1, 25))
        self.tagged = Node(0, 52, 0, tags={'key': 'value'})

    def test___repr__(self):
        expect(repr(self.bare)) == \
            'Node(0, 52.0, 0.0, False, None, None, None)'
        expect(repr(self.named)) == \
            ("Node(0, 52.0, 0.0, True, 'jnrowe', "
             'Timestamp(2008, 1, 25, 0, 0), None)')
        expect(repr(self.tagged)) == \
            "Node(0, 52.0, 0.0, False, None, None, {'key': 'value'})"

    def test___str__(self):
        expect(str(self.bare)) == """Node 0 (52°00'00"N, 000°00'00"E)"""
        expect(str(self.named)) == \
            ("""Node 0 (52°00'00"N, 000°00'00"E) [visible, user: jnrowe, """
             'timestamp: 2008-01-25T00:00:00+00:00]')
        expect(str(self.tagged)) == \
            """Node 0 (52°00'00"N, 000°00'00"E) [key: value]"""

    def test_toosm(self):
        xml_str_compare('<node id="0" lat="52.0" lon="0.0" visible="false"/>',
                        etree.tostring(self.bare.toosm()))
        xml_str_compare(
            '<node id="0" lat="52.0" lon="0.0" timestamp="2008-01-25T00:00:00+00:00" user="jnrowe" visible="true"/>',
            etree.tostring(self.named.toosm()))
        xml_str_compare(
            '<node id="0" lat="52.0" lon="0.0" visible="false"><tag k="key" v="value"/></node>',
            etree.tostring(self.tagged.toosm()))

    def test_get_area_url(self):
        expect(self.bare.get_area_url(3)) == \
            'http://api.openstreetmap.org/api/0.5/map?bbox=%s,%s,%s,%s' \
            % (-0.04384973831146972, 51.97300340214557, 0.04384973831146972,
               52.02699659785445)
        expect(self.bare.get_area_url(12)) == \
            'http://api.openstreetmap.org/api/0.5/map?bbox=%s,%s,%s,%s' \
            % (-0.1753986342770412, 51.892013608582225, 0.1753986342770412,
               52.10798639141778)

    def test_fetch_area_osm(self):
        # FIXME: The following test is skipped, because the Osm object doesn't
        # support a reliable way __repr__ method.
        #expect(Home.fetch_area_osm(3)
        pass


class TestWay(TestCase):
    def setUp(self):
        self.bare = Way(0, (0, 1, 2))
        self.named = Way(0, (0, 1, 2), True, 'jnrowe',
                         utils.Timestamp(2008, 1, 25))
        self.tagged = Way(0, (0, 1, 2), tags={'key': 'value'})

    def test___repr__(self):
        expect(repr(self.bare)) == 'Way(0, [0, 1, 2], False, None, None, None)'
        expect(repr(self.named)) == \
            ("Way(0, [0, 1, 2], True, 'jnrowe', Timestamp(2008, 1, 25, 0, 0), "
             'None)')
        expect(repr(self.tagged)) == \
            "Way(0, [0, 1, 2], False, None, None, {'key': 'value'})"

    def test___str__(self):
        expect(str(self.bare)) == 'Way 0 (nodes: 0, 1, 2)'
        expect(str(self.named)) == \
            ('Way 0 (nodes: 0, 1, 2) [visible, user: jnrowe, timestamp: '
             '2008-01-25T00:00:00+00:00]')
        expect(str(self.tagged)) == 'Way 0 (nodes: 0, 1, 2) [key: value]'
        nodes = [
            Node(0, 52.015749, -0.221765, True, 'jnrowe',
                 utils.Timestamp(2008, 1, 25, 12, 52, 11), None),
            Node(1, 52.015761, -0.221767, True, None,
                 utils.Timestamp(2008, 1, 25, 12, 53, 14),
                 {'created_by': 'hand', 'highway': 'crossing'}),
            Node(2, 52.015754, -0.221766, True, 'jnrowe',
                 utils.Timestamp(2008, 1, 25, 12, 52, 30),
                 {'amenity': 'pub'}),
        ]
        data = self.tagged.__str__(nodes).splitlines()
        expect(data[0]) == 'Way 0 [key: value]'
        expect(data[1]) == \
            ("""    Node 0 (52°00'56"N, 000°13'18"W) [visible, user: """
             'jnrowe, timestamp: 2008-01-25T12:52:11+00:00]')
        expect(data[2]) == \
            ("""    Node 1 (52°00'56"N, 000°13'18"W) [visible, timestamp: """
             '2008-01-25T12:53:14+00:00, created_by: hand, highway: crossing]')
        expect(data[3]) == \
            ("""    Node 2 (52°00'56"N, 000°13'18"W) [visible, user: """
             'jnrowe, timestamp: 2008-01-25T12:52:30+00:00, amenity: pub]')

    def test_toosm(self):
        xml_str_compare(
            '<way id="0" visible="false"><nd ref="0"/><nd ref="1"/><nd ref="2"/></way>',
            etree.tostring(self.bare.toosm())
        )
        xml_str_compare(
            '<way id="0" timestamp="2008-01-25T00:00:00+00:00" user="jnrowe" visible="true"><nd ref="0"/><nd ref="1"/><nd ref="2"/></way>',
            etree.tostring(self.named.toosm()))
        xml_str_compare(
            '<way id="0" visible="false"><tag k="key" v="value"/><nd ref="0"/><nd ref="1"/><nd ref="2"/></way>',
            etree.tostring(self.tagged.toosm()))

class TestOsm(TestCase):
    def setUp(self):
        self.region = Osm(open('tests/data/osm'))

    def test_import_locations(self):
        data = list(map(str, sorted([x for x in self.region if isinstance(x, Node)],
                                    key=lambda x: x.ident)))
        expect(data[0]) == \
            ("""Node 0 (52°00'56"N, 000°13'18"W) [visible, user: jnrowe, """
             'timestamp: 2008-01-25T12:52:11+00:00]')
        expect(data[1]) == \
            ("""Node 1 (52°00'56"N, 000°13'18"W) [visible, timestamp: """
             '2008-01-25T12:53:00+00:00, created_by: hand, highway: crossing]')
        expect(data[2]) == \
            ("""Node 2 (52°00'56"N, 000°13'18"W) [visible, user: jnrowe, """
             'timestamp: 2008-01-25T12:52:30+00:00, amenity: pub]')

    def test_export_osm_file(self):
        export = self.region.export_osm_file()
        osm_xml = etree.parse('tests/data/osm')
        for e1, e2 in zip(export.getiterator(), osm_xml.getiterator()):
            xml_compare(e1, e2)
            # expect(e1.tag) == e2.tag
            # expect(e1.text) == e2.text
            # expect(e1.attrib) == e2.attrib



#! /usr/bin/python -tt
# coding=utf-8
"""setup.py - Setuptools tasks and config for upoints"""
# Copyright © 2007-2014  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from setuptools import setup

# Hack to import _version file without importing upoints/__init__.py, its
# purpose is to allow import without requiring dependencies at this point.
_version = {}
execfile('upoints/_version.py', {}, _version)

install_requires = list(map(str.strip,
                            open('extra/requirements.txt').readlines()))

setup(
    name='upoints',
    version=_version['dotted'],
    description='Modules for working with points on Earth',
    long_description=open('README.rst').read(),
    author='James Rowe',
    author_email='jnrowe@gmail.com',
    url='https://github.com/JNRowe/upoints/',
    license='GPL-3',
    keywords=['baken', 'cities', 'geonames', 'gis', 'gps', 'gpx', 'navigation',
              'nmea', 'openstreetmap', 'trigpointing', 'weather', 'xearth'],
    packages=['upoints', ],
    include_package_data=True,
    package_data={},
    entry_points={'console_scripts': ['edist = upoints.edist:main', ]},
    zip_safe=False,
    classifiers=[
        'Development Status :: 7 - Inactive',
        'Environment :: Console',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Database',
        'Topic :: Scientific/Engineering :: GIS',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Text Processing :: Filters',
        'Topic :: Text Processing :: Markup :: XML',
        'Topic :: Utilities',
    ],
    obsoletes=['earth_distance'],
    install_requires=install_requires,
    tests_require=['nose2', 'expecter'],
)
